/******************************************************************************/
/* ST_CTYPE.C								      */
/******************************************************************************/

/* ------------------------------------------------------------------------- */
/*
 * tctype.c - TOS ctype validation
 * Copyright  1992 Atari Corporation
 * ===================================
 *
 * 920225 kbad
 * 930503 Pradip K Fatehpuria
 */
/* ------------------------------------------------------------------------- */

#include "st_def.h"
#include "st_ctype.h"

/* ------------------------------------------------------------------------- */

/* Special characters which could be used in a field name */

unsigned int	TCT_LEX_[] =
	{
/*   0 */	0,0,0,0 , 0,0,0,0 , 0,0,0,0 , 0,0,0,0 ,
/*  16 */	0,0,0,0 , 0,0,0,0 , 0,0,0,0 , 0,0,0,0 ,
/*  32 */	0,0,0,8 , 0,0,0,8 , 0,0,0,0 , 0,0,0,0 ,
/*  48 */	2,2,2,2 , 2,2,2,2 , 2,2,8,8 , 0,0,0,8 ,
/*  64 */	8,4,4,4 , 4,4,4,4 , 4,4,4,4 , 4,4,4,4 ,
/*  80 */	4,4,4,4 , 4,4,4,4 , 4,4,4,8 , 0,8,0,8 ,
/*  96 */	8,4,4,4 , 4,4,4,4 , 4,4,4,4 , 4,4,4,4 ,
/* 112 */	4,4,4,4 , 4,4,4,4 , 4,4,4,8 , 8,8,0,0 ,
/* 128 */	4,4,4,4 , 4,4,4,4 , 4,4,4,4 , 4,4,4,4 ,
/* 144 */	4,4,4,4 , 4,4,4,4 , 4,4,4,8 , 8,8,4,4 ,
/* 160 */	4,4,4,4 , 4,4,4,4 , 8,8,8,8 , 8,8,8,8 ,
/* 176 */	4,4,4,4 , 4,4,4,4 , 4,8,8,8 , 8,8,8,8 ,
/* 192 */	4,4,8,8 , 8,8,8,8 , 8,8,8,8 , 8,8,8,8 ,
/* 208 */	8,8,8,8 , 8,8,8,8 , 8,8,8,8 , 8,8,8,8 ,
/* 224 */	8,8,8,8 , 8,8,8,8 , 8,8,8,8 , 8,8,8,8 ,
/* 240 */	0,0,0,0 , 0,0,0,0 , 8,8,8,0 , 8,8,8,8
	};

/* TOS ctype table : defines the type of different characters */

unsigned int	TCT_[] =
	{
	0,		/* [EOF] -1	*/

	CC_,		/* [NUL] 0	*/
	CC_,		/* [^A]	1	*/
	CC_,		/* [^B]	2	*/
	CC_,		/* [^C]	3	*/
	CC_,		/* [^D]	4	*/
	CC_,		/* [^E]	5	*/
	CC_,		/* [^F]	6	*/
	CC_,		/* [^G]	7	*/
	CC_,		/* [^H]	8	*/
	CC_|WS_,	/* [^I]	9	*/
	CC_|WS_,	/* [^J]	10	*/
	CC_|WS_,	/* [^K]	11	*/
	CC_|WS_,	/* [^L]	12	*/
	CC_|WS_,	/* [^M]	13	*/
	CC_,		/* [^N]	14	*/
	CC_,		/* [^O]	15	*/
	CC_,		/* [^P]	16	*/
	CC_,		/* [^Q]	17	*/
	CC_,		/* [^R]	18	*/
	CC_,		/* [^S]	19	*/
	CC_,		/* [^T]	20	*/
	CC_,		/* [^U]	21	*/
	CC_,		/* [^V]	22	*/
	CC_,		/* [^W]	23	*/
	CC_,		/* [^X]	24	*/
	CC_,		/* [^Y]	25	*/
	CC_,		/* [^Z]	26	*/
	CC_,		/* [^[]	27	*/
	CC_,		/* [^\]	28	*/
	CC_,		/* [^]]	29	*/
	CC_,		/* [^^]	30	*/
	CC_,		/* [^_]	31	*/

	SP_|WS_,	/* [ ]	32	*/

	PU_,		/* [!]	33	*/
	PU_,		/* ["]	34	*/
	PU_,		/* [#]	35	*/
	PU_,		/* [$]	36	*/
	PU_,		/* [%]	37	*/
	PU_,		/* [&]	38	*/
	PU_,		/* [']	39	*/
	PU_,		/* [(]	40	*/
	PU_,		/* [)]	41	*/
	WC_,		/* [*]	42	*/
	PU_,		/* [+]	43	*/
	PU_,		/* [,]	44	*/
	PU_,		/* [-]	45	*/
	PC_,		/* [.]	46	*/
	NF_,		/* [/]	47	*/

	NU_|HX_,	/* [0]	48	*/
	NU_|HX_,	/* [1]	49	*/
	NU_|HX_,	/* [2]	50	*/
	NU_|HX_,	/* [3]	51	*/
	NU_|HX_,	/* [4]	52	*/
	NU_|HX_,	/* [5]	53	*/
	NU_|HX_,	/* [6]	54	*/
	NU_|HX_,	/* [7]	55	*/
	NU_|HX_,	/* [8]	56	*/
	NU_|HX_,	/* [9]	57	*/

	PC_,		/* [:]	58	*/
	PU_,		/* [;]	59	*/
	PU_,		/* [<]	60	*/
	PU_,		/* [=]	61	*/
	PU_,		/* [>]	62	*/
	WC_,		/* [?]	63	*/
	PU_,		/* [@]	64	*/

	UC_|HX_,	/* [A]	65	*/
	UC_|HX_,	/* [B]	66	*/
	UC_|HX_,	/* [C]	67	*/
	UC_|HX_,	/* [D]	68	*/
	UC_|HX_,	/* [E]	69	*/
	UC_|HX_,	/* [F]	70	*/
	UC_,		/* [G]	71	*/
	UC_,		/* [H]	72	*/
	UC_,		/* [I]	73	*/
	UC_,		/* [J]	74	*/
	UC_,		/* [K]	75	*/
	UC_,		/* [L]	76	*/
	UC_,		/* [M]	77	*/
	UC_,		/* [N]	78	*/
	UC_,		/* [O]	79	*/
	UC_,		/* [P]	80	*/
	UC_,		/* [Q]	81	*/
	UC_,		/* [R]	82	*/
	UC_,		/* [S]	83	*/
	UC_,		/* [T]	84	*/
	UC_,		/* [U]	85	*/
	UC_,		/* [V]	86	*/
	UC_,		/* [W]	87	*/
	UC_,		/* [X]	88	*/
	UC_,		/* [Y]	89	*/
	UC_,		/* [Z]	90	*/

	PU_,		/* [[]	91	*/
	PU_,		/* [\]	92	*/
	PU_,		/* []]	93	*/
	PU_,		/* [^]	94	*/
	PU_,		/* [_]	95	*/
	PU_,		/* [`]	96	*/

	LC_|HX_,	/* [a]	97	*/
	LC_|HX_,	/* [b]	98	*/
	LC_|HX_,	/* [c]	99	*/
	LC_|HX_,	/* [d]	100	*/
	LC_|HX_,	/* [e]	101	*/
	LC_|HX_,	/* [f]	102	*/
	LC_,		/* [g]	103	*/
	LC_,		/* [h]	104	*/
	LC_,		/* [i]	105	*/
	LC_,		/* [j]	106	*/
	LC_,		/* [k]	107	*/
	LC_,		/* [l]	108	*/
	LC_,		/* [m]	109	*/
	LC_,		/* [n]	110	*/
	LC_,		/* [o]	111	*/
	LC_,		/* [p]	112	*/
	LC_,		/* [q]	113	*/
	LC_,		/* [r]	114	*/
	LC_,		/* [s]	115	*/
	LC_,		/* [t]	116	*/
	LC_,		/* [u]	117	*/
	LC_,		/* [v]	118	*/
	LC_,		/* [w]	119	*/
	LC_,		/* [x]	120	*/
	LC_,		/* [y]	121	*/
	LC_,		/* [z]	122	*/

	PU_,		/* [{]	123	*/
	PU_,		/* [|]	124	*/
	PU_,		/* [}]	125	*/
	PU_,		/* [~]	126	*/

	CC_,		/* [DEL] 127	*/

	U2_,		/* []	128	*/
	L2_,		/* []	129	*/
	L2_,		/* []	130	*/
	L2_,		/* []	131	*/
	L2_,		/* []	132	*/
	L2_,		/* []	133	*/
	L2_,		/* []	134	*/
	L2_,		/* []	135	*/
	L2_,		/* []	136	*/
	L2_,		/* []	137	*/
	L2_,		/* []	138	*/
	L2_,		/* []	139	*/
	L2_,		/* []	140	*/
	L2_,		/* []	141	*/
	U2_,		/* []	142	*/
	U2_,		/* []	143	*/
	U2_,		/* []	144	*/
	L2_,		/* []	145	*/
	U2_,		/* []	146	*/
	L2_,		/* []	147	*/
	L2_,		/* []	148	*/
	L2_,		/* []	149	*/
	L2_,		/* []	150	*/
	L2_,		/* []	151	*/
	L2_,		/* []	152	*/
	U2_,		/* []	153	*/
	U2_,		/* []	154	*/

	SY_,		/* []	155	*/
	SY_,		/* []	156	*/
	SY_,		/* []	157	*/

	U2_,		/* []	158	*/
	L2_,		/* []	159	*/
	L2_,		/* []	160	*/
	L2_,		/* []	161	*/
	L2_,		/* []	162	*/
	L2_,		/* []	163	*/
	L2_,		/* []	164	*/
	U2_,		/* []	165	*/
	L2_,		/* []	166	*/
	L2_,		/* []	167	*/

	SY_,		/* []	168	*/
	SY_,		/* []	169	*/
	SY_,		/* []	170	*/
	SY_,		/* []	171	*/
	SY_,		/* []	172	*/
	SY_,		/* []	173	*/
	SY_,		/* []	174	*/
	SY_,		/* []	175	*/

	L2_,		/* []	176	*/
	L2_,		/* []	177	*/
	U2_,		/* []	178	*/
	L2_,		/* []	179	*/
	L2_,		/* []	180	*/
	U2_,		/* []	181	*/
	U2_,		/* []	182	*/
	U2_,		/* []	183	*/
	U2_,		/* []	184	*/

	SY_,		/* []	185	*/
	SY_,		/* []	186	*/
	SY_,		/* []	187	*/
	SY_,		/* []	188	*/
	SY_,		/* []	189	*/
	SY_,		/* []	190	*/
	SY_,		/* []	191	*/

	L2_,		/* []	192	*/
	U2_,		/* []	193	*/

	HB_,		/* []	194	*/
	HB_,		/* []	195	*/
	HB_,		/* []	196	*/
	HB_,		/* []	197	*/
	HB_,		/* []	198	*/
	HB_,		/* []	199	*/
	HB_,		/* []	200	*/
	HB_,		/* []	201	*/
	HB_,		/* []	202	*/
	HB_,		/* []	203	*/
	HB_,		/* []	204	*/
	HB_,		/* []	205	*/
	HB_,		/* []	206	*/
	HB_,		/* []	207	*/
	HB_,		/* []	208	*/
	HB_,		/* []	209	*/
	HB_,		/* []	210	*/
	HB_,		/* []	211	*/
	HB_,		/* []	212	*/
	HB_,		/* []	213	*/
	HB_,		/* []	214	*/
	HB_,		/* []	215	*/
	HB_,		/* []	216	*/
	HB_,		/* []	217	*/
	HB_,		/* []	218	*/
	HB_,		/* []	219	*/
	HB_,		/* []	220	*/

	SY_,		/* []	221	*/
	SY_,		/* []	222	*/
	SY_,		/* []	223	*/
	SY_,		/* []	224	*/
	SY_,		/* []	225	*/
	SY_,		/* []	226	*/
	SY_,		/* []	227	*/
	SY_,		/* []	228	*/
	SY_,		/* []	229	*/
	SY_,		/* []	230	*/
	SY_,		/* []	231	*/
	SY_,		/* []	232	*/
	SY_,		/* []	233	*/
	SY_,		/* []	234	*/
	SY_,		/* []	235	*/
	SY_,		/* []	236	*/
	SY_,		/* []	237	*/
	SY_,		/* []	238	*/
	SY_,		/* []	239	*/
	SY_,		/* []	240	*/
	SY_,		/* []	241	*/
	SY_,		/* []	242	*/
	SY_,		/* []	243	*/
	SY_,		/* []	244	*/
	SY_,		/* []	245	*/
	SY_,		/* []	246	*/
	SY_,		/* []	247	*/
	SY_,		/* []	248	*/
	SY_,		/* []	249	*/
	SY_,		/* []	250	*/
	SY_,		/* []	251	*/
	SY_,		/* []	252	*/
	SY_,		/* []	253	*/
	SY_,		/* []	254	*/
	SY_		/* [ ]	255	*/
	};

/* ------------------------------------------------------------------------- */

static	int	Txform( int, char * );

/******************************************************************************/
/******************************************************************************/

#ifdef	_NO_TCTYPE_MACROS

#undef	isalnum(c)
#undef	isalpha(c)
#undef	iscntrl(c)
#undef	iscsym(c)
#undef	iscsymf(c)
#undef	isdigit(c)
#undef	isgraph(c)
#undef	islower(c)
#undef	ispunct(c)
#undef	isprint(c)
#undef	isspace(c)
#undef	isupper(c)
#undef	isxdigit(c)

int	isalnum(c)
int	c;
{
	return(TCT_[c+1] & ALNUM_);
}

int	isalpha(c)
int	c;
{
	return(TCT_[c+1] & (UC_ | LC_));
}

int	iscntrl(c)
int	c;
{
	return(TCT_[c+1] & CC_);
}

int	iscsym(c)
int	c;
{
	return((TCT_[c+1] & ALNUM_) || toascii(c)=='_');
}

int	iscsymf(c)
int	c;
{
	return((TCT_[c+1] & (UC_ | LC_)) || toascii(c)=='_');
}

int	isdigit(c)
int	c;
{
	return(TCT_[c+1] & NU_);
}

int	isgraph(c)
int	c;
{
	return(TCT_[c+1] & (PUNC_ | ALNUM_ | XASCII_));
}

int	islower(c)
int	c;
{
	return(TCT_[c+1] & LC_);
}

int	ispunct(c)
int	c;
{
	return(TCT_[c+1] & PUNC_);
}

int	isprint(c)
int	c;
{
	return(TCT_[c+1] & (PUNC_ | ALNUM_ | XASCII_ | SP_));
}

int	isspace(c)
int	c;
{
	return(TCT_[c+1] & WS_);
}

int	isupper(c)
int	c;
{
	return(TCT_[c+1] & UC_);
}

int	isxdigit(c)
int	c;
{
	return(TCT_[c+1] & HX_);
}

#endif

/******************************************************************************/
/******************************************************************************/

/* ANSI functions */

int	tolower(c)
register int	c;
{
	return((isupper(c)) ? _tolower(c) : c);
}

int	toupper(c)
register int	c;
{
	return((islower(c)) ? _toupper(c) : c);
}

/* -----------------------------------------------------------------------------
 * Transform a character to its partner in a sorted lookup table
 */

static	int	Txform(c, map)
register int 	c;
register char	*map;
{
register int	mc;

	while (c > (mc = (unsigned char)(*map)))
	{
		if (c == mc)
		{
			return((int)(*map));
		}
		map++, map++;
	}
	return(c);
}

/* -----------------------------------------------------------------------------
 * Transform a TOS character to lowercase
 */

int	Tolower(c)
register int	c;
{

/* ASCII-sorted int'l upper->lower case pairs */
/* static unsigned char map2lower[] = ""; */

static unsigned char map2lower[] = "\
\x80\x87\x8e\x84\x8f\x86\x90\x82\x92\x91\x99\x94\x9a\x81\
\xa5\xa4\xb2\xb3\xb5\xb4\xb6\x85\xb7\xb0\xb8\xb1\xc1\xc0";

	if (islower(c))
	{
		return(_tolower(c));
	}
	return(Txform(c, map2lower));
}

/* -----------------------------------------------------------------------------
 * Transform a TOS character to uppercase
 */

int	Toupper(c)
register int	c;
{

/* ASCII-sorted int'l lower->uppper case pairs */
/* static unsigned char map2upper[] = ""; */

static unsigned char map2upper[] = "\
\x81\x9a\x82\x90\x84\x8e\x85\xb6\x86\x8f\x87\x80\x91\x92\
\x94\x99\xa4\xa5\xb0\xb7\xb1\xb8\xb3\xb2\xb4\xb5\xc0\xc1";

	if (isupper(c))
	{
		return(_toupper(c));
	}
	return(Txform(c, map2upper));
}

/* -----------------------------------------------------------------------------
 * Validate a TEDINFO character, return valid char or 0
 */

int	Tvalid(c, template)
register int	c;
register int	template;
{
	switch (template)
	{
	case	'9': /* digit : [0-9] */

		return(isTdigit(c) ? c : 0);

	case	'A': /* upper-alpha : [ A-Z-] */

		if (!isTalpha(c))
		{
			return(0);
		}
		break;

	case	'N': /* upper-alnum : [ 0-9A-Z-] */

		if (!isTalnum(c))
		{
			return(0);
		}
		break;

	case	'P': /* wild path : [!-.0-~-(255)] */

		return(isTwildpath(c) ? c : 0);

	case	'p': /* path spec : [!-)+,\-0->@-~-(255)] */

		return(isTpathspec(c) ? c : 0);

	case	'F': /* wild file : [!-\-0-9;-~-(255)] */

		return(isTwildfile(c) ? c : 0);

	case	'f': /* file name : [!-)+,\-0-9;->@-~-(255)] */

		return(isTfilename(c) ? c : 0);

	case	'a': /* alpha : [ A-Za-z----] */

		return(isTalpha(c) ? c : 0);

	case	'n': /* alnum : [ 0-9A-Za-z----] */

		return(isTalnum(c) ? c : 0);

	case	'x': /* upper any : . */
		break;

	default: /* 'X' et al */

		return(c);
	}

	return(Toupper(c));
}

/******************************************************************************/
/* End of ST_CTYPE.C							      */
/******************************************************************************/
