/******************************************************************************/
/* ST_HARD.C								      */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <osbind.h>
#include <aes.h>

#include <sys\types.h>
#include <sys\stat.h>

#include "st_aes.h"
#include "st_str.h"
#include "st_hard.h"
#include "cd_host.h"
#include "cdmonusa.h"

/* -------------------------------------------------------------------------- */

#define	MY_LRWABS	0

#define	SEEKBUFF	1

#define	ONEREADS	1
#define	TWOREADS	0

/* -------------------------------------------------------------------------- */
/*
 * File Control Block
 * as it appears on the disk.
 */

#define FA_RO		0x01		/* file may not be deleted/changed */
#define FA_HIDDEN	0x02		/* these two may be reversed */
#define FA_SYSTEM	0x04
#define FA_LABEL	0x08		/* volume label */
#define FA_SUBDIR	0x10		/* file is a subdirectory */
#define FA_ARCHIVE	0x20		/* archive bit */

#define FA_ALL		0xF7		/* all but label bit */
#define FA_NORM		0xE7		/* all but label & subdir */
#define	FA_NOTFSEL	0x08		/* entries with these bits are not going to be listed */

#define ERASED		((char)0xe5)

typedef	struct	__fcb
{
	unsigned char	f_name[11];
	unsigned char	f_attrib;
	unsigned char	f_fill[10];
	unsigned int	f_time;
	unsigned int	f_date;
	unsigned int	f_clust;
	unsigned long	f_fileln;
} FCB;

/*
 * BIOS Parameter Block
 * Describes a volume's organization.
 */

typedef	struct	__bpb
{
	unsigned int	recsiz;		/* record size in bytes */
	unsigned int	clsiz;		/* #records/cluster */
	unsigned int 	clsizb;		/* #bytes/cluster (recsiz * clsiz) */
	unsigned int	rdlen;		/* root directory length in records */
	unsigned int	fsiz;		/* fat size in records */
	unsigned int	fatrec;		/* first fat record (of last fat) */
	unsigned int	datrec;		/* first data record */
	unsigned int	numcl;		/* number of data clusters available */
	unsigned int	b_flags;	/* bit 0 = 0 for 12-bit fat, 1 for 16-bit */
} BPB;

/* -------------------------------------------------------------------------- */

#define	DOS_SIGPOS	0x1fe		/* MSDOS boot sector's signature position */
#define	DOS_SIG		0x55aa		/* Signature for valid MSDOS boot secttor */
#define	DOS_PM		0x1be		/* MSDOS boot sector's partition map */
#define	DOS_PMTYPE	0x04		/* offset to type of MSDOS partition */
#define	DOS_PMSTART	0x08		/* offset to start of partition */
#define	DOS_PMLENGTH	0x0C		/* offset to length of partition */

/* -------------------------------------------------------------------------- */
/* symbolic constants */

#define TRUE	1
#define FALSE	0

#define	MAX_PATHNAME	130		/* maximum pathname length */
#define	MAX_FILENAME	14		/* maximum filename length */

/* -------------------------------------------------------------------------- */

#define	MAX_FRMSIZE	2352

/* -------------------------------------------------------------------------- */

#define	TePtext(y)  (unsigned char *)((TEDINFO *)Tree[(y)].ob_spec)->te_ptext

#define	NO_ITEMS	8
#define	RSC_PATHLENGTH	40

/* -------------------------------------------------------------------------- */

#define	DEF_DEVNO	10		/* physical device no */
#define	SIZ_ROOTSECTOR	512		/* size of root sector */

#define	MAX_FATLENGTH	0x20000L	/* maximum 128K (64K clusters) */

#define	FSEL_BUFFSIZE	0x4000		/* buffer used in cd_host.s (in bytes) */

/* -------------------------------------------------------------------------- */

extern	unsigned long	Lswap( void *, unsigned long );
extern	unsigned int	Wswap( void *, unsigned long );

/* -------------------------------------------------------------------------- */

static	void	fs_fp2fn( unsigned char *, unsigned char * );
static	void	fs_fn2fp( unsigned char *, unsigned char * );

static	long	fs_dirsize( unsigned int );
static	long	fs_readfcb( unsigned int, FCB *, long * );

/* -------------------------------------------------------------------------- */

static	unsigned int	dev_no;				/* device number */
static	unsigned char	dev_rootsec[ SIZ_ROOTSECTOR ];	/* size of a root sector */
static	unsigned char	dev_parttype;			/* type of partition */
static	unsigned long	dev_partstart;			/* starting sector no of partition */
static	unsigned long	dev_partlen;			/* length of partition */
static	BPB	dev_bpb;				/* BPB of scsi device */

static	unsigned char	FAT_table[ MAX_FATLENGTH ];	/* current FAT table as read from disk */

/* -------------------------------------------------------------------------- */

#if MY_LRWABS > 0

#undef	DEF_DEVNO
#define	DEF_DEVNO	0

#undef	Lrwabs
extern	long Lrwabs(int, void *, int, int, long);

#endif

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

int	fsel_initio(scsino)
register unsigned int	scsino;
{
register long	err;
register unsigned long	numcl;
register unsigned int	rdlen;

/* set the device number as specified */

	dev_no = DEF_DEVNO + scsino;

/* read the root sector on drive */

	err = Lrwabs(0xa, dev_rootsec, 1, dev_no, 0);
	if (err)
		return(FALSE);

	if ((*(unsigned int *)(&dev_rootsec[ DOS_SIGPOS ])) != DOS_SIG)
		return(FALSE);

	dev_parttype = (unsigned char)dev_rootsec[ DOS_PM + DOS_PMTYPE ];
	if ((dev_parttype != 5) && (dev_parttype != 6))
		return(FALSE);

	dev_partstart = Lswap(dev_rootsec, (unsigned long)(DOS_PM + DOS_PMSTART));
	if (dev_partstart == 0l)
		return(FALSE);

	dev_partlen = Lswap(dev_rootsec, (unsigned long)(DOS_PM + DOS_PMLENGTH));
	if (dev_partlen == 0l)
		return(FALSE);

/* read boot sector of drive */

	err = Lrwabs(0xa, dev_rootsec, 1, dev_no, dev_partstart);
	if (err)
		return(FALSE);

	dev_bpb.recsiz = Wswap(dev_rootsec, 0x0Bl);
	dev_bpb.clsiz = (unsigned int)dev_rootsec[ 0x0D ];
	dev_bpb.clsizb = dev_bpb.recsiz * dev_bpb.clsiz;

	rdlen = Wswap(dev_rootsec, 0x11l) * 32;
	dev_bpb.rdlen = rdlen / dev_bpb.recsiz;
	dev_bpb.rdlen += ((rdlen % dev_bpb.recsiz) ? 1 : 0);

	dev_bpb.fsiz = Wswap(dev_rootsec, 0x16l);
	dev_bpb.fatrec = Wswap(dev_rootsec, 0x0El) + dev_bpb.fsiz;
	dev_bpb.datrec = dev_bpb.fatrec + dev_bpb.fsiz + dev_bpb.rdlen;

	numcl = (unsigned long)Wswap(dev_rootsec, 0x13l);
	if (!numcl) numcl = Lswap(dev_rootsec, 0x20l);
	numcl -= dev_bpb.datrec;
	dev_bpb.numcl = (unsigned int)(numcl / dev_bpb.clsiz);

	dev_bpb.b_flags = 1;	/* 16 bit fat */

	err = Lrwabs(0xa, FAT_table, dev_bpb.fsiz, dev_no, dev_partstart + dev_bpb.fatrec);
	return(err ? FALSE : TRUE);
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

static	void	fs_fp2fn(fpname, filename)
register unsigned char	*fpname;
register unsigned char	*filename;
{
register int	i, j;

	i = j = 0;
	while ((i < 8) && (fpname[i] != ' '))
		filename[j++] = fpname[i++];

	i = 8;
	while ((i < 11) && (fpname[i] != ' '))
	{
		if (i == 8) filename[j++] = '.';
		filename[j++] = fpname[i++];
	}
	filename[j] = '\0';
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	fs_fn2fp(fpname, filename)
register unsigned char	*fpname;
register unsigned char	*filename;
{
register int	i, j;

	i = j = 0;
	while (fpname[i] && (fpname[i] != '.'))
		filename[j++] = fpname[i++];
	while (j < 8)
		filename[j++] = ' ';
	if (fpname[i] == '.')
	{
		i++;
		while (fpname[i])
			filename[j++] = fpname[i++];
	}
	while (j < 11)
		filename[j++] = ' ';
	filename[j] = '\0';
	return;
}

/* -----------------------------------------------------------------------------
 */

static	long	fs_dirsize(startcl)
register unsigned int	startcl;
{
register unsigned int	cl;
register long	len;

	if (!(cl = startcl))
	{
		/* root directory */
		return((long)dev_bpb.rdlen * (long)dev_bpb.recsiz);
	}
	/* otherwise go through the FAT table and find the directory length */
	for (len = 1L ;; len ++)
	{
		cl = Wswap(FAT_table, (unsigned long)cl + (unsigned long)cl);

		if (!cl)				return(E_FSELBADFAT);
		if ((cl >= 0xFFF0) && (cl <= 0xFFF7))	return(E_FSELBADFAT);
		if ((cl >= 0xFFF8) && (cl <= 0xFFFF))	return(len * dev_bpb.clsizb);
		if (cl >= dev_bpb.numcl)		return(E_FSELBADFAT);
	}
}

/* -----------------------------------------------------------------------------
 */

static	long	fs_readfcb(startcl, fcb, maxfcbs)
register unsigned int	startcl;
register FCB	*fcb;
register long	*maxfcbs;
{
register FCB	*nxt;
register unsigned int	cl;
register unsigned int	count;
register long	recno;
register long	i;
register long	maxno;
register long	fcbcnt;
register long	err;

	fcbcnt = 0;
	for (cl = startcl ;;)
	{
		if (!cl)
		{
			count = dev_bpb.rdlen;
			recno = (long)dev_bpb.fatrec + (long)dev_bpb.fsiz;
		}
		else
		{
			count = dev_bpb.clsiz;
			recno = (long)dev_bpb.datrec + (long)(cl - 2) * (long)dev_bpb.clsiz;
		}
		recno += dev_partstart;
		err = Lrwabs(0xa, fcb, count, dev_no, recno);
		if (err)
			return(err);

		i = 0;
		maxno = (long)(((long)count * (long)dev_bpb.recsiz) / sizeof(FCB));
		while (i < maxno)
		{
			if (fcb->f_name[0] == 0x00)	break;
			if (fcb->f_name[0] == ERASED)	break;
			if (fcb->f_attrib & FA_NOTFSEL)	break;
			fcbcnt ++;
			fcb ++;
			i ++;
		}
		nxt = fcb;
		while (i < maxno)
		{
			if (nxt->f_name[0] == 0x00)	break;
			if (nxt->f_name[0] == ERASED)	{ nxt ++; i ++; continue; }
			if (nxt->f_attrib & FA_NOTFSEL)	{ nxt ++; i ++; continue; }
			st_memcpy((unsigned char *)fcb, (unsigned char *)nxt, sizeof(FCB));
			fcbcnt ++;
			fcb ++;
			nxt ++;
			i ++;
		}
		if (!cl)
			break;

		cl = Wswap(FAT_table, (unsigned long)cl + (unsigned long)cl);
		if ((cl >= 0xFFF8) && (cl <= 0xFFFF))
			break;
	}
	(*maxfcbs) = fcbcnt;
	return(0l);
}

/* -----------------------------------------------------------------------------
 */

long	fsel_file(devno, retname, dta)
unsigned int	devno;
unsigned char	retname[];
FSELDTA	*dta;
{
static	unsigned int	fsel_flag = FALSE;
static	unsigned int	fsel_devno = 0xFFFF;
static	unsigned char	fs_pathname[ MAX_PATHNAME ];
static	unsigned char	fs_filename[ MAX_FILENAME ];
static	unsigned int	fs_startcl;

static	int	SlideRsc[] = { FSELSLID, FSELBOX, FSELUP, FSELDOWN };
static	int	FuncRsc[] = { FSELLST0, FSELLST1, FSELLST2, FSELLST3, FSELLST4, FSELLST5, FSELLST6, FSELLST7, -1};

register unsigned char	*cp;
register FCB	*dirfcb;
register unsigned int	currcl;
register int	newdir;

register OBJECT	*Tree;
register int	Respons;
register long	maxno;
register long	ind;
register long	sel;
register long	oldind;
int	i;
long	diff;
unsigned char	pathname[ MAX_PATHNAME ];
unsigned char	filename[ MAX_FILENAME ];
GRECT	box;

#if MY_LRWABS
void	*savessp;
long	ret;
#endif
	if ((fsel_devno != devno) || !fsel_flag)
	{
		fsel_flag = TRUE;
		fsel_devno = devno;
		strcpy(fs_pathname, "\\*.*");
		strcpy(fs_filename, "");
		fs_startcl = 0;
	}
#if MY_LRWABS
	savessp = Super(NULL);
	ret = fsel_initio(devno);
	Super(savessp);
	if (!ret)
		return(E_FSELINIT);
#else
	if (!fsel_initio(devno))
		return(E_FSELINIT);
#endif

	strcpy(pathname, fs_pathname);
	st_str_ncpy(filename, fs_filename, 11);
	currcl = fs_startcl;

	Tree = st_objc_get_tree(FSELLIST);
	st_objc_form_center(Tree, &box);

	Tree[ FSELPATH ].ob_flags &= ~EDITABLE;
	strcpy(TePtext(FSELPATH), "");
	strcpy(TePtext(FSELFILE), "");
	st_listdial_init(Tree, FuncRsc);
	Tree[ FSELBOX ].ob_y = 0;
	st_objc_draw_tree(Tree);

	dirfcb = NULL;
	newdir = 0x0007;
	for (;;)
	{
		if (newdir & 0x0001)
		{
			st_str_ncpy(TePtext(FSELPATH), pathname, RSC_PATHLENGTH);
			objc_draw(Tree, FSELPTBX, MAX_DEPTH, box.g_x, box.g_y, box.g_w, box.g_h);
			newdir &= ~0x0001;
		}
		if (newdir & 0x0002)
		{
			st_str_ncpy(TePtext(FSELFILE), filename, 11);
			objc_draw(Tree, FSELFLBX, MAX_DEPTH, box.g_x, box.g_y, box.g_w, box.g_h);
			newdir &= ~0x0002;
		}
		if (newdir & 0x0004)
		{
			long	err;
			long	maxfcbs;

			if (dirfcb != NULL)
				free(dirfcb);
#if MY_LRWABS
			savessp = Super(NULL);
			err = fs_dirsize(currcl);
			Super(savessp);
#else
			err = fs_dirsize(currcl);
#endif
			if (err < 0)
			{
				st_objc_erase_tree(Tree);
				return(err);
			}
			dirfcb = (FCB *)calloc(1, err);
			if (dirfcb == NULL)
			{
				st_objc_erase_tree(Tree);
				return(E_FSELMEM);
			}
#if MY_LRWABS
			savessp = Super(NULL);
			err = fs_readfcb(currcl, dirfcb, &maxfcbs);
			Super(savessp);
#else
			err = fs_readfcb(currcl, dirfcb, &maxfcbs);
#endif
			if (err)
			{
				free(dirfcb);
				st_objc_erase_tree(Tree);
				return(err);
			}
			maxno = maxfcbs;

			oldind = -1;
			ind = 0;
			sel = -1;
			newdir &= ~0x0004;
		}

		if (ind != oldind)
		{
			unsigned char	attr;
			unsigned char	*t;
			long	j;

			st_listdial_init(Tree, FuncRsc);
			for (i = 0, j = ind ; (i < NO_ITEMS) && (j < maxno) ; i++, j++)
			{
				attr = dirfcb[j].f_attrib;
				t = TePtext(FuncRsc[i]);
				t[0] = ((attr & FA_SUBDIR) ? 0x07 : ((attr & FA_RO) ? 0x7F : 0x20));
				st_str_ncpy(&t[1], dirfcb[j].f_name, 11);

				Tree[ FuncRsc[i] ].ob_state = NORMAL;
				Tree[ FuncRsc[i] ].ob_flags |= (TOUCHEXIT | SELECTABLE);
				if (j == sel) Tree[ FuncRsc[i] ].ob_state = SELECTED;
			}

			objc_draw(Tree, FSELPARA, MAX_DEPTH, box.g_x, box.g_y, box.g_w, box.g_h);
			st_listdial_updslider(Tree, SlideRsc, ind, maxno, NO_ITEMS);
			oldind = ind;
		}

		Respons = st_listdial_form_do(Tree, FSELFILE);
		Respons &= 0x7FFF;

		if ((Respons == FSELOK) || (Respons == FSELCAN))
		{
			Tree[ Respons ].ob_state &= ~SELECTED;
			break;
		}

		if (diff = st_listdial_scroll(Tree, Respons, SlideRsc, ind, maxno, NO_ITEMS, 1))
		{
			ind += diff;
			continue;
		}

		if ((diff = st_listdial_select(Respons, FuncRsc)) >= 0)
		{
			unsigned char	newname[ MAX_FILENAME ];
			unsigned char	wildcard[ MAX_FILENAME ];

			sel = ind + diff;
			if (dirfcb[ sel ].f_attrib & FA_SUBDIR)
			{
				fs_fp2fn(dirfcb[ sel ].f_name, newname);

				cp = strrchr(pathname, '\\');
				strcpy(wildcard, cp);
				(*cp) = '\0';
				if (!strcmp(newname, "."))
				{
					strcat(pathname, wildcard);
				}
				else
				if (!strcmp(newname, ".."))
				{
					cp = strrchr(pathname, '\\');
					if (cp == NULL) cp = pathname;
					strcpy(cp, wildcard);
				}
				else
				{
					strcat(pathname, "\\");
					strcat(pathname, newname);
					strcat(pathname, wildcard);
				}
				currcl = Wswap(&(dirfcb[ sel ].f_clust), 0l);
				strcpy(filename, "");
				newdir |= 0x0007;
			}
			else
			{
				st_str_ncpy(filename, dirfcb[ sel ].f_name, 11);
				newdir |= 0x0002;
			}
		}
	}
	st_objc_erase_tree(Tree);

	st_str_toupper(pathname);
	st_str_toupper(filename);

	strcpy(fs_pathname, pathname);
	strcpy(fs_filename, filename);
	fs_startcl = currcl;

	if ((Respons == FSELCAN) ||
	    !strlen(filename) ||
	    !strcmp(filename, ".") ||
	    strchr(filename, '?') ||
	    strchr(filename, '*') ||
	    (sel < 0))
	{
		if (dirfcb != NULL)
			free(dirfcb);
		return(0);
	}

	strcpy(retname, pathname);
	cp = strrchr(retname, '\\');
	fs_fp2fn(filename, pathname);
	strcpy(cp+1,pathname);

	dta->d_fat   = Wswap(&(dirfcb[ sel ].f_clust), 0l);
	dta->d_fattr = (unsigned int)dirfcb[ sel ].f_attrib;
	dta->d_tandd = Lswap(&(dirfcb[ sel ].f_time), 0l);
	dta->d_fsize = Lswap(&(dirfcb[ sel ].f_fileln), 0l);
	strcpy(dta->d_fname, pathname);

	if (dirfcb != NULL)
		free(dirfcb);
	return(1);
}

/* -----------------------------------------------------------------------------
 */

long	fsel_findfile(pathname, dta)
unsigned char	pathname[];
FSELDTA	*dta;
{
register unsigned char	*cp;
register unsigned char	*nextcp;
register long	i;
register long	maxno;
unsigned char	lookfp[ MAX_FILENAME ];

register FCB	*dirfcb;
register unsigned int	currcl;
register long	err;
register long	dirlen;
long	maxfcbs;

	cp = strchr(pathname, '\\');
	if (cp == NULL)
	{
		return(0l);
	}
	currcl = 0;
	dirfcb = NULL;
	for (;;)
	{
		if (dirfcb != NULL)
			free(dirfcb);

		dirlen = fs_dirsize(currcl);
		if (dirlen < 0)
			return(dirlen);

		dirfcb = (FCB *)calloc(1, dirlen);
		if (dirfcb == NULL)
			return(E_FSELMEM);

		err = fs_readfcb(currcl, dirfcb, &maxfcbs);
		if (err)
		{
			free(dirfcb);
			return(err);
		}
		maxno = maxfcbs;

		cp ++;
		nextcp = strchr(cp, '\\');
		if (nextcp)	(*nextcp) = '\0';
		fs_fn2fp(cp, lookfp);
		if (nextcp)	(*nextcp) = '\\';
		cp = nextcp;

		for (i = 0 ; i < maxno ; i++)
		{
			if (!strncmp(dirfcb[i].f_name, lookfp, 11))
				break;
		}
		if (i >= maxno)
		{
			/* could not find the file */
			return(0l);
		}
		currcl = Wswap(&(dirfcb[i].f_clust), 0l);
		if (cp)
		{
			/* sub-directory found */
			continue;
		}

		/* file found */
		dta->d_fat   = Wswap(&(dirfcb[i].f_clust), 0l);
		dta->d_fattr = (unsigned int)dirfcb[i].f_attrib;
		dta->d_tandd = Lswap(&(dirfcb[i].f_time), 0l);
		dta->d_fsize = Lswap(&(dirfcb[i].f_fileln), 0l);
		strcpy(dta->d_fname, dirfcb[i].f_name);

		if (dirfcb != NULL)
			free(dirfcb);
		return(1l);
	}
}

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

/* We have to load a buffer full of data from the file. Depending upon */
/* start of playing back in a file the buffer may be loaded from one or more */
/* clusters */

/* Whenever a file is to be loaded from the disk fsel_initfiledata(...) is */
/* called to initialise the file data - like starting position and the length */
/* of the file.  This routine will return the length of the maximum buffer to */
/* be filled on success.  On failure, this routine will return a negative */
/* number.  Now, to load the buffer we call fsel_readfiledata in succession */
/* till the buffer is filled from data from one or more files, depending upon */
/* the size of the files */

/* In order to fill the buffer with zero data (like data padded at the */
/* begining and end of tracks/sessions.  First we call fsel_initzerodata(...) */
/* with starting position and length of the zero data to be filled.  Then we */
/* will call fsel_readzerodata(....) in succession till the buffer is filled */

/* In order to fill one full buffer of data, the above mentioned routines may */
/* be called once or more, depending upon the size of the buffer itself, the */
/* size of different files on the disc and the length of the padding between */
/* tracks and sessions */

/* Variable p_ptinbuf keeps track of where in the buffer we are at any */
/* point of time */

/* Also, we assume that each file is multiple of frame in length.  In order */
/* to do that we may have to padd zeroes at the end of the file.  Whenever */
/* a buffer gets filled before we padd a file with zeroes, then play_howmny0 */
/* is set to the number of extra zeroes to be passed in the next instance we */
/* have to fill the buffer */

static	unsigned int	p_currcls;	/* current cluster to be loaded */

/*==============================================*/
#if ONEREADS > 0
/*==============================================*/

static	unsigned long	p_filepos;	/* current file position to read */

/*==============================================*/
#endif
/*==============================================*/

static	unsigned long	p_currpos;	/* current position in file */
static	unsigned long	p_currlen;	/* length of file to be loaded */

static	unsigned int	p_ptinbuf;	/* important - current position in buffer */

/* -----------------------------------------------------------------------------
 */

int	fsel_initbuff()
{
	p_ptinbuf = 0;
	return(FSEL_BUFFSIZE);
}

/******************************************************************************/
/******************************************************************************/

/*==============================================*/
#if TWOREADS > 0
/*==============================================*/

/* -----------------------------------------------------------------------------
 */

int	fsel_initfiledata(cl, findpos, findlen)
register unsigned int	cl;
register unsigned long	findpos;
register unsigned long	findlen;
{
register unsigned long	currpos = 0l;

	for (;;)
	{
		currpos += (unsigned long)dev_bpb.clsizb;
		if (currpos > findlen) currpos = findlen;

		/* check if the starting position in this cluster */
		if (currpos > findpos)
		{
			p_currpos = findpos;
			p_currlen = findlen;
			p_currcls = cl;
			return(1);
		}

		/* get the next cluster address */
		cl = Wswap(FAT_table, (unsigned long)cl * 2l);
	}
}

/* -----------------------------------------------------------------------------
 */

int	fsel_readfiledata()
{
register BPB	*bpb = &(dev_bpb);
register unsigned int	ptinbuf = p_ptinbuf;
register unsigned int	maxno;
register unsigned long	limit;
register unsigned int	temp;
register unsigned int	offset_rec;
register unsigned int	offset_pos;
register unsigned int	i;

	/* if pointer in the buffer has reached its limit, we will */
	/* reset it back to the start of the buffer */
	if (ptinbuf >= FSEL_BUFFSIZE) ptinbuf = 0;

	limit = p_currpos % (unsigned long)(bpb->clsizb);
	maxno = (unsigned int)limit;				/* byte position in cluster */
	offset_rec = maxno / bpb->recsiz;			/* sector position in cluster */
	offset_pos = maxno % bpb->recsiz;			/* byte position in sector */
	A_rdind = (unsigned long)offset_pos;
	A_maxno = 0;

	/* fill the rest of the buffer */
	for (i = 0 ;;)
	{
		maxno = FSEL_BUFFSIZE - ptinbuf;		/* number of bytes yet to be filled */
		if (!maxno) break;				/* check if the buffer is full */

		limit = p_currlen - p_currpos;
		if (maxno > limit) maxno = (unsigned int)limit;	/* only read till the end of file */

		maxno += offset_pos;				/* part of the first sector to be read */
		temp = maxno % bpb->recsiz;			/* part of the last sector to be read */
		maxno /= bpb->recsiz;				/* number of sectors yet to be read */
		if (temp) maxno ++;				/* whether part of last sector to be read */

		temp = bpb->clsiz - offset_rec;			/* number of sectors left in cluster */
		if (maxno > temp) maxno = temp;			/* align to cluster boundary */

		/* get the starting sector number for the current cluster */
		limit = dev_partstart + (unsigned long)(bpb->datrec) + (unsigned long)(p_currcls - 2) * (unsigned long)(bpb->clsiz);
		if (Lrwabs(0xa, A_data+i, maxno, dev_no, limit + offset_rec))
			return(E_FSELREADERR);

		offset_rec += maxno;				/* new sector position in cluster */
		maxno *= bpb->recsiz;				/* number of bytes read from disk */
		i += maxno;					/* update the A buffer pointer */

		maxno -= offset_pos;				/* take out extra bytes at the begining of the sector */
		ptinbuf += maxno;				/* new fill position in buffer */
		if (ptinbuf > FSEL_BUFFSIZE)
		{
			maxno -= (ptinbuf - FSEL_BUFFSIZE);
			ptinbuf = FSEL_BUFFSIZE;
		}
		p_currpos += (unsigned long)maxno;		/* new position in file */
		A_maxno += maxno;				/* number of bytes to send */

		if (p_currpos >= p_currlen) break;		/* see if the file is fully read */

		/* get the next cluster if crossed cluster boundary */ 
		if (offset_rec >= bpb->clsiz)
		{
			p_currcls = Wswap(FAT_table, (unsigned long)p_currcls * 2l);
			offset_rec = 0;
		}
		offset_pos = 0;
	}
	p_ptinbuf = ptinbuf;

	if (maxno = A_maxno) SndBufA();
	return(maxno);
}

/*==============================================*/
#endif
/*==============================================*/

/*==============================================*/
#if ONEREADS > 0
/*==============================================*/

/* -----------------------------------------------------------------------------
 */

int	fsel_initfiledata(cl, findpos, findlen)
register unsigned int	cl;
register unsigned long	findpos;
register unsigned long	findlen;
{
register unsigned long	currpos = 0l;
register unsigned long	filepos;

	for (;;)
	{
		filepos = currpos;
		currpos += (unsigned long)dev_bpb.clsizb;

		/* check if the starting position in this cluster */
		if (currpos > findpos)
		{
			p_filepos = filepos;
			p_currpos = findpos;
			p_currlen = findlen;
			p_currcls = cl;

			A_rdind = findpos - filepos;
			while (A_rdind >= FSEL_BUFFSIZE)
			{
				p_filepos += FSEL_BUFFSIZE;
				A_rdind -= FSEL_BUFFSIZE;
			}
			return(1);
		}
		/* get the next cluster address */
		cl = Wswap(FAT_table, (unsigned long)cl * 2l);
	}
}

/* -----------------------------------------------------------------------------
 */

int	fsel_readfiledata()
{
register BPB	*bpb = &(dev_bpb);
register unsigned long	filepos = p_filepos;
register unsigned int	ptinbuf = p_ptinbuf;
register unsigned long	recno;
register unsigned int	maxno;
register unsigned long	limit;

	/* if pointer in the buffer has reached its limit, we will */
	/* reset it back to the start of the buffer */
	if (ptinbuf >= FSEL_BUFFSIZE) ptinbuf = 0;

	/* read data from file if necessary */
	if (filepos < p_currlen)
	{
		/* get the starting sector number of the cluster */
		recno  = dev_partstart + (unsigned long)(bpb->datrec) + (unsigned long)(p_currcls - 2) * (unsigned long)bpb->clsiz;

		/* get the starting sector number and read data */
		maxno = (unsigned int)(filepos % (unsigned long)(bpb->clsizb));
		maxno /= bpb->recsiz;
		if (Lrwabs(0xa, A_data+A_ldind, (FSEL_BUFFSIZE / bpb->recsiz), dev_no, recno + (unsigned long)maxno))
			return(E_FSELREADERR);

		/* update read index in the buffer */
		A_ldind = ((A_ldind == 0) ? FSEL_BUFFSIZE : 0);

		/* update the read position in the file */
		filepos += FSEL_BUFFSIZE;
		if (filepos > p_currlen) filepos = p_currlen;

		/* switch to next cluster, if crossed cluster boundary */
		if (!(filepos % (unsigned long)bpb->clsizb))
		{
			p_currcls = Wswap(FAT_table, (unsigned long)p_currcls * 2l);
		}
	}
	/* number of bytes yet to be sent to DSP */
	maxno = FSEL_BUFFSIZE - ptinbuf;
	limit = filepos - p_currpos;
	if ((unsigned long)maxno > limit) maxno = (unsigned int)limit;

	p_filepos = filepos;
	p_currpos += (unsigned long)maxno;
	p_ptinbuf = ptinbuf + maxno;

	if (A_maxno = maxno) SndBufA();
	return(maxno);
}

/*==============================================*/
#endif
/*==============================================*/

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

int	fsel_initzerodata(findpos, findlen)
unsigned long	findpos;
unsigned long	findlen;
{
	p_currlen = findlen - findpos;
	return(1);
}

/* -----------------------------------------------------------------------------
 */

int	fsel_readzerodata()
{
register unsigned int	ptinbuf = p_ptinbuf;
register unsigned long	currlen = p_currlen;
register unsigned int	maxno;

	/* if pointer in the buffer has reached its limit, we will */
	/* reset it back to the start of the buffer */
	if (ptinbuf >= FSEL_BUFFSIZE) ptinbuf = 0;

	maxno = FSEL_BUFFSIZE - ptinbuf;			/* amount needed to fill one buffer */
	if (maxno > currlen) maxno = (unsigned int)currlen;	/* maximum number of zeroes to be sent */
	p_currlen = currlen - (unsigned long)maxno;		/* length of zero beffer yet to be sent */
	p_ptinbuf = ptinbuf + maxno;				/* amount of buffer filled */
	if (B_maxno = maxno) SndBufB();				/* how many zeroes to be sent to Host Port */
	return(maxno);
}

/******************************************************************************/
/******************************************************************************/

/*==============================================*/
#if SEEKBUFF > 0
/*==============================================*/

/* -----------------------------------------------------------------------------
 */

long	fsel_seekreadfile(cl, findpos, findlen)
register unsigned int	cl;
register unsigned long	findpos;
register unsigned long	findlen;
{
register BPB	*bpb = &(dev_bpb);
register unsigned long	old_ldind;
register unsigned long	currpos;
register unsigned long	filepos;

	for (currpos = 0l ;;)
	{
		filepos = currpos;
		currpos += (unsigned long)(bpb->clsizb);

		/* check if the starting position is in this cluster */
		if (currpos > findpos)
		{
			while (filepos <= findpos) filepos += FSEL_BUFFSIZE;
			filepos -= FSEL_BUFFSIZE;
			break;
		}

		/* get the next cluster address */
		cl = Wswap(FAT_table, (unsigned long)cl * 2l);
	}
	old_ldind = C_ldind;
	while (C_ldind < C_limit)
	{
		/* get the starting sector number and read data */
		currpos  = (unsigned long)(dev_partstart + (unsigned long)(bpb->datrec));
		currpos += (unsigned long)((unsigned long)(cl - 2) * (unsigned long)bpb->clsiz);
		currpos += (unsigned long)((filepos % (unsigned long)(bpb->clsizb)) / bpb->recsiz);
		if (Lrwabs(0xa, C_data+C_ldind, (FSEL_BUFFSIZE / bpb->recsiz), dev_no, currpos))
			return(E_FSELREADERR);

		/* update read index in the buffer and read position in file */
		C_ldind += FSEL_BUFFSIZE;

		filepos += FSEL_BUFFSIZE;
		if (filepos >= findlen)
			break;

		/* switch to next cluster, if crossed cluster boundary */
		if (!(filepos % (unsigned long)(bpb->clsizb)))
		{
			cl = Wswap(FAT_table, (unsigned long)cl * 2l);
		}
	}
	return(C_ldind - old_ldind);
}

/* -----------------------------------------------------------------------------
 */

long	fsel_seekreadzero(findpos, findlen)
unsigned long	findpos;
unsigned long	findlen;
{
register unsigned char	*data = C_data + C_ldind;
register long	maxno;
register long	limit;
register long	i;

	maxno = C_limit - C_ldind;
	limit = findlen - findpos;
	if (maxno > limit) maxno = limit;

	i = 0l;
	while (i < maxno) { *data++ = 0; i++; }
	C_ldind += maxno;
	return(maxno);
}

/* -----------------------------------------------------------------------------
 */

int	fsel_seeksenddata()
{
register unsigned int	ptinbuf = p_ptinbuf;
register unsigned int	maxno;
register unsigned long	limit;

	/* if pointer in the buffer has reached its limit, we will */
	/* reset it back to the start of the buffer */
	if (ptinbuf >= FSEL_BUFFSIZE) ptinbuf = 0;

	maxno = FSEL_BUFFSIZE - ptinbuf;		/* amount needed to fill one buffer */
	limit = C_ldind - C_rdind;
	if (maxno > limit) maxno = (unsigned int)limit;	/* maximum number of zeroes to be sent */
	p_ptinbuf = ptinbuf + maxno;			/* amount of buffer filled */
	if (C_maxno = maxno) SndBufC();			/* how many zeroes to be sent to Host Port */
	return(maxno);
}

/*==============================================*/
#endif
/*==============================================*/

/******************************************************************************/
/* End of ST_HARD.C							      */
/******************************************************************************/
