/******************************************************************************/
/* ST_SCRAP.C								      */
/******************************************************************************/

#include "st_def.h"
#include "st_struc.h"
#include "st_aes.h"
#include "st_bind.h"
#include "st_scrap.h"
#include "cd_bind.h"

/* -------------------------------------------------------------------------- */

extern	int	gl_scrhandle;
extern	int	gl_gemhandle;

extern	unsigned char	gl_pathname[];

extern	CDROM	*gl_actdocu;

/* -------------------------------------------------------------------------- */

static	unsigned char	scrap_pathname[ MAX_PATHNAME ];

static	CDROM	*scrap_clipdptr = NULL;
static	int	scrap_cliptype = CLP;

static	unsigned char	*gl_extension = { "TOC" };

/* -------------------------------------------------------------------------- */

static	int	st_clip_getscrapdir( unsigned char * );
static	int	st_clip_setscrapdir( unsigned char * );

static	void	st_clip_scrapfile( unsigned char *, unsigned char *, unsigned char * );

static	int	st_clip_scrapclear( unsigned char * );
static	int	st_clip_getfilename( unsigned char * );

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

int	st_clip_initscrap()
{
unsigned char	scrappath[ MAX_PATHNAME ];

/* get the old scrap folder name */

	if (st_clip_getscrapdir(scrappath))
	{
		strcpy(scrap_pathname, scrappath);
		return(TRUE);
	}

/* no old scrap sxists. so, set the new default scrap folder */

/* does drive C exist? */
/* if so, assume it's home of the Clipboard Directory */
/* if not, then assume drive A: */
/* yikes!  a floppy-based system running WORKS? */

	strcpy(scrappath, ((Drvmap() & 4) ? "C" : "A"));
	strcat(scrappath, ":\\CLIPBRD\\");

	strcpy(scrap_pathname, scrappath);
	return(st_clip_setscrapdir(scrappath));
}

/* -----------------------------------------------------------------------------
 * This routine checks if a scrap directory exists.  If it does then process
 * the scrap directory path.  If any error in thisscrap path name string or
 * if the scrap directory does not exist, return the default scrap directory
 * path name.
 */

static	int	st_clip_getscrapdir(scrappath)
register unsigned char	scrappath[];
{
struct	stat	statbuf;
register unsigned char	*cp;
register int	i;
register int	len;
register long	drivemap;
register int	drvmask;
register int	drvno;

/* no scrap directory exists, return FALSE */

	if (!scrp_read(scrappath))
		return(FALSE);

/* remove spaces, if any, from clipboard path */
/* also get rid of the last backslash from the scrap folder name */

	st_str_squeeze(scrappath);
	len = (int)strlen(scrappath) - 1;
	if (scrappath[ len ] == '\\')
		scrappath[ len ] = '\0';

/* check if the scrap directory set to the root directroy, like "X:\" */
/* if so, then check if the logical drive exists or not */

	len = (int)strlen(scrappath);
	if (len < 2)
		return(FALSE);

	drivemap = Drvmap();
	drvno = scrappath[0] - 'A';
	drvmask = 1;
	for (i = 0 ; i < drvno ; i++)
		drvmask <<= 1;

	if (!(drivemap & drvmask))
		return(FALSE);

	if (len > 2)
	{

/* error, that means the scrap folder does not exist or it is empty */

		if (stat(scrappath, &statbuf) == -1)
			return(FALSE);

		if (!(statbuf.st_mode & S_IJDIR))
		{

/* the scrap string ends with a filename. so get rid of the last token */
/* in the pathname and establish an existing scrap folder */

			cp = strrchr(scrappath, '\\');
			(*cp) = '\0';
		}
	}

/* reset the scrap path and return error if could not */

	strcat(scrappath, "\\");
	return(scrp_write(scrappath) ? TRUE : FALSE);
}

/* -----------------------------------------------------------------------------
 * Set a new clipboard folder with the scrap manager
 */

static	int	st_clip_setscrapdir(scrappath)
register unsigned char	scrappath[];
{
struct	stat	statbuf;
register int	i;
register int	len;
register long	drivemap;
register long	drvmask;
register int	drvno;

/* get rid of the trailing '\' from the scrap folder name */

	len = (int)strlen(scrappath) - 1;
	if (scrappath[ len ] == '\\')
		scrappath[ len ] = '\0';

/* the scrap folder should atleast be "X:\" */

	len = (int)strlen(scrappath);
	if (len < 2)
		return(FALSE);

/* prepare the drive mask for the requested scrap folder */
/* if the scrap folder in the root folder then accept it */
/* if logical drive exists. */
/* otherwise check for the validity of the scrap folder */
/* default clipboard drive does not exist, then return */

	drivemap = Drvmap();
	drvno = scrappath[0] - 'A';
	drvmask = 1;
	for (i = 0 ; i < drvno ; i++)
		drvmask <<= 1;

	if (!(drivemap & drvmask))
		return(FALSE);

	if (len > 2)
	{
		if (!stat(scrappath, &statbuf))
		{

/* it is not a valid folder, then return */

			if (!(statbuf.st_mode & 0x10))
				return(FALSE);

/* default scrap folder exists */
		}
		else
		{
/* create the scrap folder - if could not create, then return */

			if (Dcreate(scrappath) < 0)
				return(FALSE);

/* default scrap folder is created. continue... */
		}
	}

/* set the scrap folder with the AES */
/* if could not set the default scrap folder then return */

	strcat(scrappath, "\\");
	return(scrp_write(scrappath) ? TRUE : FALSE);
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

static	void	st_clip_scrapfile(scrapfile, scrappath, extension)
unsigned char	scrapfile[];
unsigned char	scrappath[];
unsigned char	extension[];
{
	strcpy(scrapfile, scrappath);
	strcat(scrapfile, "SCRAP.");
	strcat(scrapfile, extension);
	return;
}

/* -----------------------------------------------------------------------------
 * Initialises clipboard for "copy" operation.
 */

static	int	st_clip_scrapclear(scrappath)
register unsigned char	scrappath[];
{
register DMABUFFER *theDTA;
register int	err;
unsigned char	scrapfile[ MAX_PATHNAME ];

/* get the old scrap folder name */
/* if no old scrap exists, then set the new default scrap folder */

	if (!st_clip_getscrapdir(scrappath))
	{
		strcpy(scrappath, scrap_pathname);
		if (!st_clip_setscrapdir(scrappath))
			return(FALSE);
	}

/* get the DTA address */

	theDTA = (DMABUFFER *)Fgetdta();

/* if special clipboard operations then clear the current */
/* alternate buffer to be used for copying, */
/* otherwise clear the normal clipboard buffer */

	st_clip_scrapfile(scrapfile, scrappath, "*");

/* look for the scrap files in the scrap folders and delete them */

	err = (int)Fsfirst(scrapfile, 2);
	while (!err)
	{
		strcpy(scrapfile, scrappath);
		strcat(scrapfile, theDTA->d_fname);
		Fdelete(scrapfile);

		err = (int)Fsnext();
	}
	return(TRUE);
}

/* -----------------------------------------------------------------------------
 * Find the clipboard contents. Looks for specific type of data in the scrap
 * directory.  Returns the path to the clipboard file. Returns TRUE if there
 * is any content in the clipboard, otherwise returns FALSE
 */

static	int	st_clip_getfilename(s)
register unsigned char	*s;
{
register DMABUFFER *theDTA;
unsigned char	scrappath[ MAX_PATHNAME ];
unsigned char	scrapfile[ MAX_PATHNAME ];

	if (!st_clip_getscrapdir(scrappath))
		return(FALSE);

	theDTA = (DMABUFFER *)Fgetdta();
	st_clip_scrapfile(scrapfile, scrappath, gl_extension);
	if (Fsfirst(scrapfile, 0))
		return(FALSE);

	strcpy(s, scrappath);
	strcat(s, theDTA->d_fname);
	return(TRUE);
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * Sets the Clipboard type as given.
 * and the clipboard contents to the given buffer pointer.
 */

void	st_clip_setpointer()
{
register FILE	*fp;
register int	ret;
unsigned char	scrappath[ MAX_PATHNAME ];
unsigned char	scrapfile[ MAX_PATHNAME ];

	if (!st_clip_scrapclear(scrappath))
		goto out;

/* prepare "SCRAP.TXT" for global copy operations */

	st_clip_scrapfile(scrapfile, scrappath, "TXT");

/* open the clipboard text file */

	if (!(fp = fopen(scrapfile, "wb")))
		goto out;

	ret = cd_clipexport(fp);
	fclose(fp);
	if (!ret)
	{
		Fdelete(scrapfile);
		goto out;
	}

/* prepare a CDROM type of clipboard file name */

	st_clip_scrapfile(scrapfile, scrappath, gl_extension);

/* open the clipboard file */

	if (!(fp = fopen(scrapfile, "wb")))
		goto out;

/* write the clipboard on to the scrap file */

	ret = cd_clipsave(fp);
	fclose(fp);
	if (!ret)
	{
		Fdelete(scrapfile);
		goto out;
	}
	return;

out :	st_error_message(E_FATALSCRAP);
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * Returns the current clipboard type.
 */

int	st_clip_gettype()
{
unsigned char	scrapfile[ MAX_PATHNAME ];

	return(st_clip_getfilename(scrapfile));
}

/* -----------------------------------------------------------------------------
 * Returns the current Clipboard buffer pointer.
 */

CDROM	*st_clip_getpointer()
{
register FILE	*fp;
register CDROM	*dptr;
int	ret;
unsigned char	scrapfile[ MAX_PATHNAME ];

	st_clip_freeafterpaste();
	if (!st_clip_getfilename(scrapfile))
		return(NULL);

	if (!(fp = fopen(scrapfile, "rb")))
		goto out;

	scrap_clipdptr = dptr = cd_clipload(fp, &ret);
	fclose(fp);
	if (ret)
	{
		if (ret != 1)
			goto out;
		st_error_message(E_MALL);
	}
	return(dptr);

out :	st_error_message(E_FATALSCRAP);
	return(NULL);
}

/* -----------------------------------------------------------------------------
 */

void	st_clip_freeafterpaste()
{
	if (scrap_clipdptr == NULL)
		return;

	cd_clipfree(scrap_clipdptr);
	scrap_clipdptr = NULL;
	return;
}

/******************************************************************************/
/* End of ST_SCRAP.C							      */
/******************************************************************************/
