;**************************************************************************
; (C)1993 ATARI CORP.       SECRET & CONFIDENTIAL       ALL RIGHTS RESERVED
;
;
;				CD_code3.s
;
;	This MC68K file contains a program that demonstrates the use of 
;	some of the subroutines presently available to drive the Jaguar 
;	CD Module.  The program demonstrates loading and executing code 
;	stored on CD.  All calls to the CD Module are made as "bsrs".
;
;	Note that the this demo requires the DSP program JeriN68K.gas.
;
;	The makefile is CD_code3.mak.
;
;	The hardware assumptions here are a CDROM interface card Rev 2.0
;	mounted in the 2 MB Sylvester Development System.
;
;	The demo disc must be 17 NOV 93 DMS, or another that contains an
;	executable file as track 1 of the disc.  The executable itself
;	must be assembled to execute from address $C000.  Track 1 must be 
;	mastered with header and tail per Atari specs.
;
;
;				REVISION HISTORY
;
; REV.	DATE	   BY	DESCRIPTION OF EDIT
; """"	""""	   ""	"""""""""""""""""""
; 4.02	07 DEC	   PKF	Plays CD - 02 DEC 93 DMS (Alternate Pix/Audio tracks)
;****************************************************************************

;*==========================================================================*

	.include 'jaguar.inc'		; cast in concrete for console
	.include 'joypad.inc'
	.include 'cd_inits.inc'		; CD-related equates

;*==========================================================================*

	.extern	screnmem
	.extern	end_pix			; needed for GPU program load by DB
	.extern	end_aud			; needed for GPU program load by DB

	.extern	edges

;*==========================================================================*

	.globl	Opentrak
	.globl	Do_video
	.globl	Do_audio
	.globl	ChekTrak
	.globl	Clostrak

;*==========================================================================*

; These define the main CD data buf area in Jerri SRAM
; edit the values both here & in the .GAS files

Buf1_beg	equ	$F1C000		; start of buffer in Jerry
Halfbuf1	equ	$F1C0C0		; halfway address in Jerry's buffer 1

halfflg1	equ	$F1C308		; address of buf 1 half-full flag
fullflg1	equ	$F1C30C		; address of buf 1 full flag
DSP_stop	equ	$F1C310		; DSP semaphore location from M68K

Joy1		equ	$F14000		; joy1

;****************************************************************************
;****************************************************************************

;*==========================================================================*
;*==========================================================================*

Opentrak:
	movem.l	a0-a3/d0-d6,-(sp)

;					*****************************************
;					* Initialize Cd-variables		*
;					*****************************************
	clr.l	d0
	move.l	d0,CD_base

	move.l	d0,CD_found
	move.l	d0,CD_stat	
	move.l	d0,CD_error
	move.l	d0,CDcutlnL
	move.l	d0,CDcutlnH
	move.l	d0,CD_track
	move.l	d0,CD_index
	move.l	d0,CD_Rmins
	move.l	d0,CD_Rsecs
	move.l	d0,CD_Amins
	move.l	d0,CD_Asecs
	move.l	d0,CD_Afrms
	move.l	d0,CD_setup
	move.l	d0,CDTOCbeg
	move.l	d0,CDTOCend
	move.l	d0,CDTOCmin
	move.l	d0,CDTOCsec
	move.l	d0,CDTOCfrm
	move.l	d0,CDseshns
	move.l	d0,CDcutnum
	move.l	d0,CDcutCAB
	move.l	d0,CDcutstm
	move.l	d0,CDcutsts
	move.l	d0,CDcutstf
	move.l	d0,CD_DAC

	move.l	d0,CD_wrd0	

	move.l	d0,CDfatals
	move.l	d0,CDmaybes

	move.l	d0,CD_retry
	move.l	d0,PS_flag	

	move.l	d0,Init_cut
	move.l	d0,CD_mode	
	move.l	d0,DAC_mode

	move.w	d0,Strt_min
	move.w	d0,Strt_sec
	move.w	d0,strt_frm
	move.w	d0,Stop_min
	move.w	d0,Stop_sec
	move.w	d0,Stop_frm

	move.w	d0,Waittime
	move.w	d0,Session

;					*****************************************
;					* Initialise DRAM buffer pointers	*
;					*****************************************

.initdata:
	move.l	#screnmem,d0
	move.l	d0,Blok_ptr
	move.l	d0,Blok_beg
	add.l	#160000,d0
	move.l	d0,Blok_end

;					*****************************************
;					* Clear our DRAM data buffer		*
;					*****************************************

;.clermem:
;	move.l	Blok_beg,a0		; our lowest DRAM address
;	move.l	Blok_end,d0		; number of bytes
;.clrloop:				; clear CD variables and data buffer area
;	move.l	#0,(a0)+
;	cmp.l	a0,d0
;	bne	.clrloop

;					*****************************************
;					* Get the TOC on the current CD		*
;					*****************************************

	bsr	CDmode_4
	clr.l	d4
	jsr	CDminmax
	clr.l	d4
	jsr	CDbigTOC

	movem.l	(sp)+,d0-d6/a0-a3
	rts


;*==========================================================================*
;*==========================================================================*

Do_video:
	movem.l	a0-a3/d0-d6,-(sp)

;					*****************************************
;					* Stop DSP, if it's running		*
;					*****************************************

	move.l	#1,DSP_stop		; semaphore to Jerry asks him to stop

;					*****************************************
;					* Beginning of CD_related code		*
;					*****************************************

.start_CD:
	move.l	$30(sp),CD_track	; init with requested track

;					*****************************************
;					* interrupt on every sample pair	*
;					*****************************************

.i2s:
	move.l	#$14,SMODE		; external clk, interrupt on every sample pair

;					*****************************************
;					* Load, & initialize DSP program	*
;					*****************************************

.DSP_do:
	move.l	#end_pix,a0
	move.l	(a0)+,d0		; final location of DSP code 
	move.l	d0,a1
	move.l	d0,d1			; save DSP code start in d1
	move.l	(a0)+,d0		; size in bytes
	asr.l	#2,d0
.xferloop:
	move.l	(a0)+,(a1)+
	dbra	d0,.xferloop

.DSP_init:
	move.l	d1,D_PC			; set DSP PC to start of code in SRAM
	move.l	#0,halfflg1		; clear semaphores
	move.l	#0,fullflg1
	move.l	#0,DSP_stop

;					*****************************************
;					* Setup CD and play track		*
;					*****************************************

.setup_CD:
	bsr	CDmode_2		; setup the CD Module for CD ROM mode 2

.play_1:				; very short track
	move.l	CD_track,d1		; play track #n

	jsr	Play_CD

	move.l	CD_wrd0,d0
	move.l	d0,CD_error
	move.l	#$66666666,d6		; disc drive got error
	btst	#10,d0
	bne	.Fini			; exit with error code
	move.l	#1,D_CTRL		; start DSP per CD_code1

;					*****************************************
;					* Fill up a DRAM block with CD data	*
;					*****************************************

.Blokfill:				; main data handling loop...
	move.l	Blok_beg,Blok_ptr	; initialize DRAM pointer
	move.l	Blok_end,d4		; get end point of DRAM block

.Get_data:				; loop to fetch data from DSP FIFO
	move.l	#Buf1_beg,a2		; base of FIFO in DSP
	move.l	#Halfbuf1,a3		; halfway point of FIFO in DSP
	move.w	#47,d2			; if 1/2 flag set, number of longs to get
	move.w	#47,d3			; if full flag set, number of longs to get

.MayBhalf:				; see if DSP FIFO half-buf flag is set
	move.l	#$55555555,d6		; DSP told us about an error
	move.l	DSP_stop,d0
	cmp.l	#1,d0
	beq	.Fini			; bail out on error

	move.l	halfflg1,d1
	cmp.l	#1,d1
	bne	.MayBhalf		; no, wait forever
	move.l	Blok_ptr,a1		; yes, get address of DRAM block pointer

.Get_low:				; fetch low half of FIFO contents
	move.l	(a2)+,(a1)+		; get a long
	dbra	d2,.Get_low		; do it 47 more times = 48 total
	move.l	a1,Blok_ptr		; save current position of pointer in DRAM
	move.l	#0,halfflg1		; clear half-full flag

.MayBfull:				; see if FIFO is full
	move.l	fullflg1,d1
	cmp.l	#1,d1
	bne	.MayBfull		; no, wait forever
	move.l	Blok_ptr,a1		; yes, get address of DRAM block pointer

.Get_high:				; fetch top half of FIFO contents
	move.l	(a3)+,(a1)+		; get a long
	dbra	d3,.Get_high		; do it 47 more times = 48 total
	move.l	a1,Blok_ptr		; save current position of pointer in DRAM
	move.l	#0,fullflg1		; clear full flag

.All_done:				; see if we're at the end of DRAM buffer
	cmp.l	a1,d4			; use a longword boundary
	bge	.Get_data		; keep looping till file is read

.Fini:					; gets here on an error or after playing track
	move.l	#1,DSP_stop		; semaphore to Jerry asks him to stop

	movem.l	(sp)+,d0-d6/a0-a3
	rts

;*==========================================================================*
;* DO_AUDIO	: Play audio from CD					    *
;*==========================================================================*

Do_audio:
	movem.l	a0-a3/d0-d6,-(sp)

;					*****************************************
;					* Stop DSP, if it's running		*
;					*****************************************

	move.l	#1,DSP_stop		; semaphore to Jerry asks him to stop

;					*****************************************
;					* Beginning of CD_related code		*
;					*****************************************

.start_CD:
	move.l	$30(sp),CD_track	; init with requested track

;					*****************************************
;					* Setup CD and play track		*
;					*****************************************

.setup_CD:
	bsr	CDmode_4		; setup Jerry's i2s port and clocks
				
;					*****************************************
;					* Load, initialize & run DSP program	*
;					*****************************************
.DSP_do:				; Leonard's thing 
	move.l	#end_aud,a0
	move.l	(a0)+,d0		; Final location of DSP code 
	move.l	d0,a1
	move.l	d0,d1
	move.l	(a0)+,d0		; Size in bytes
	asr.l	#2,d0
.xferloop:
	move.l	(a0)+,(a1)+
	dbra	d0,.xferloop

.DSP_init:
	move.l	d1,D_PC			; Set DSP PC to start of code in SRAM
	move.l	#0,DSP_stop

.Play_it:				; DSP's running. Now start disc

	move.l	CD_track,d1		; play track #n

;.FindTrak:
;	move.l	#99,d2			; maximum of 99 tracks
;	lea	CD_TOC,a0		; get TOC address
;
;.LookTrak:
;	move.l	(a0),d0
;	and.l	#$FF,d0
;	cmp.l	d1,d0
;	beq	.FoundTrak
;
;	add.l	#20,a0
;	sub.l	#1,d2
;	bne	.LookTrak
;
;	move.l	#$FFFFFFFF,d6		; IMPOSSIBLE - trak should have been there
;	illegal
;
;.FoundTrak:
;	move.l	#$00000000,d1		; play using time A to time B mode
;	move.l	#$00000000,d2		; initialise start time
;	move.l	#$00000000,d3		; initialise stop time
;
;	add.l	#8,a0			; point to start time of current track
;	move.l	(a0),d0			; get the start minutes of current track
;	add.l	#4,a0
;	and.l	#$000000FF,d0
;	rol.l	#8,d0
;	rol.l	#8,d0
;	or.l	d0,d2			; set start minutes of current track
;	move.l	(a0),d0			; get the start seconds of current track
;	add.l	#4,a0
;	and.l	#$000000FF,d0
;	rol.l	#8,d0
;	or.l	d0,d2			; set the start seconds of current track
;	move.l	(a0),d0			; get the start frames of current track
;	add.l	#4,a0
;	and.l	#$000000FF,d0
;	or.l	d0,d2			; set the start frames of current track
;
;	add.l	#8,a0			; get the start time of next track
;	move.l	(a0),d0			; get the start minutes of next track
;	add.l	#4,a0
;	and.l	#$000000FF,d0
;	rol.l	#8,d0
;	rol.l	#8,d0
;	or.l	d0,d3			; set start minutes of next track
;	move.l	(a0),d0			; get the start seconds of next track
;	add.l	#4,a0
;	and.l	#$000000FF,d0
;	rol.l	#8,d0
;	or.l	d0,d3			; set the start seconds of next track
;	move.l	(a0),d0			; get the start frames of next track
;	add.l	#4,a0
;	and.l	#$000000FF,d0
;	or.l	d0,d3			; set the start frames of next track

	jsr	Play_CD
	move.l	#1,D_CTRL		; set DSP GO bit to start running

;					*****************************************
;					* Leave Tom set for DRAM space & exit	*
;					*****************************************

.Fini:					; load from CD failed, so...
;	bsr	Stop_CD			; stop drive
;	move.l	#1,DSP_stop		; semaphore to tell DSP to stop itself
;	move.l	#$F00000,a0		; Tom's base register
;	move.w	#$1861,d1		; value to read DRAM space
;	move.w	d1,(a0)			; do it

	movem.l	(sp)+,d0-d6/a0-a3
	rts

;*==========================================================================*
;*==========================================================================*

ChekTrak:
	movem.l	a0-a3/d0-d6,-(sp)

	move.l	CD_track,d0
	beq	.ret

	lea	myCD_TOC,a0
	sub.l	#1,d0
	mulu.w	#12,d0
	add.l	d0,a0

	move.l	edges,d0
	btst.l	#KEY_0,d0
	beq	.GetTime
	illegal

.GetTime:

	jsr	Get_time

.ChkMins:
	move.l	(a0),d0
	move.l	CD_Amins,d1
	and.l	#$FF,d1
	cmp.l	d0,d1
	blt	.ret

.ChkSecs:
	add.l	#4,a0
	move.l	(a0),d0
	move.l	CD_Asecs,d1
	and.l	#$FF,d1
	cmp.l	d0,d1
	blt	.ret

.ChkFrms:
	add.l	#4,a0
	move.l	(a0),d0
	add.l	#4,a0
	move.l	CD_Afrms,d1
	and.l	#$FF,d1
	cmp.l	d0,d1
	blt	.ret

.PlayAgn:
	move.l	CD_track,-(sp)
	jsr	Do_audio
	add.l	#4,sp

.ret:
	movem.l	(sp)+,d0-d6/a0-a3
	rts

;*==========================================================================*
;*==========================================================================*

Clostrak:
	movem.l	a0-a3/d0-d6,-(sp)

	move.l	#1,DSP_stop		; semaphore to Jerry asks him to stop
	bsr	Stop_CD			; stop drive

	movem.l	(sp)+,d0-d6/a0-a3
	rts
;__________________________________________________________________________
;||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
;				SUBROUTINES

;				*****************************************
;				* Setup CD Module for Mode 1, CD ROM	*
;				*****************************************
CDmode_1:

.setibit:			; can't write to CD Module before setting
	move.l	#MEMCON1,a0	; the I/O bit in Tom's base register
	move.w	#$1865,d1	; value to init with to read ROM1 space
	move.w	d1,(a0)		; write to Tom

.setmotor:			; make damn sure we're Motorola-way
	move.l	#$00070007,D_END

.do_butch:
	move.l	#$0,BUTCH	; Butch enable 
	move.l	#$10000,DSCNTRL	; turn on DSA Bus
	move.l	#5,I2CNTRL	; 5 = code, f = audio 

.init_CDR:			; no interpolation, double-speed spindle
	move.w	#$150A,DS_DATA	; send to Butch
	bsr	DSA_rx		; get response

.Chekmode:			; verify CD is in mode 0A
	move.l	CD_wrd0,d0
	cmp.l	#$0000170A,d0
	bne	.init_CDR	; hang forever if CD won't get it right
	rts

;				*****************************************
;				* Setup CD Module Mode 2,CD ROM-Jerry	*
;				*****************************************
CDmode_2:

.setibit:			; can't write to CD Module before setting
	move.l	#MEMCON1,a0	; the I/O bit in Tom's base register
	move.w	#$1865,d1	; value to init with to read ROM1 space
	move.w	d1,(a0)		; write to Tom

.setmotor:			; make damn sure we're Motorola-way
	move.l	#$00070007,D_END

.do_butch:
	move.l	#$0,BUTCH	; Butch enable 
	move.l	#$10000,DSCNTRL	; turn on DSA Bus
	move.l	#$7,I2CNTRL	; i2s on, copy to jerry, forced, 16-bit master

.setmode:
	move.l	#SMODE,a0	; put address into a0
	move.l	#$14,d1		; external clk, interrupt on every sample pair
	move.l	d1,(a0)		; write to Jerry

.DSAoff2X:			; DSA in 'shut-up' mode, 2 x spindle speed
	move.w	#$150A,DS_DATA	; DSA command
	bsr	DSA_rx

.Chekmode:			; verify CD is in mode 0A
	move.l	CD_wrd0,d0
	cmp.l	#$0000170A,d0
	bne	.DSAoff2X	; hang forever if CD won't get it right
	rts

;				*****************************************
;				* Setup CD Module for Mode 4, Audio	*
;				*****************************************
CDmode_4:			; init for audio (mode 2)
.setiobit:			; can't write to CD Module before setting
	move.l	#$F00000,a0	; the I/O bit in Tom's base register
	move.w	#$1865,d1	; value to init with to read ROM1 space
	move.w	d1,(a0)		; write to Tom

.set_moto:			; make damn sure we're Motorola-way
	move.l	#$00070007,D_END

.butch:					
	move.l	#$0,BUTCH	; Butch enable
	move.l	#$10000,DSCNTRL	; turn on DSA Bus
	move.l	#$f,I2CNTRL	; 5 = code, f = audio

.make_clk:			; Init serial clk frequency to...
	move.l	#2,d1		; 2(N+1) = 6	26/6 =~ 4.3 MHz
	move.l	#$F1A150,a0	; put address of sclk reg into a0
	move.l	d1,(a0)		; write to Jerry

.set_mode:
	move.l	#$F1A154,a0	; put address into a0
	move.l	#$14,d1		; external clk, interrupt on every sample pair
	move.l	d1,(a0)		; write to Jerry
	
.interpol:			; interpolate audio data errors
	move.w	#$1501,DS_DATA	; DSA command
	bsr	DSA_rx

.setDAC:			; output audio data at 2X oversampled
	move.w	#$7002,DS_DATA	; DSA command
	bsr	DSA_rx
	rts

;				*****************************************
;				* i2s data copy to Jerry on/off		*
;				*****************************************
Jeridata:			; d1 must have a 1 or a 0 in it

.setiobit:			; can't write to CD Module before setting
	move.l	#MEMCON1,a0	; the I/O bit in Tom's base register
	move.w	#$1865,d0	; value to init with to read ROM1 space
	move.w	d0,(a0)		; write to Tom

.setmotor:			; make damn sure we're Motorola-way
	move.l	#$00070007,D_END

.test_d1:
	cmp.l	#1,d1
	beq	.Jcp_on

.Jcp_off:
	move.l	#$0,BUTCH	; Butch enable
	move.l	I2CNTRL,d0	; get contents of i2s control reg
	bclr	#0,d0		; clear bit 0
	bclr	#1,d0		; clear bit 1
	move.l	d0,I2CNTRL	; now do it
	bra	.Jcp_exit

.Jcp_on:
	move.l	#$0,BUTCH	; Butch enable
	move.l	I2CNTRL,d0	; get contents of i2s control reg
	or.l	#3,d0		; or in the Jerry copy-on bits
	move.l	d0,I2CNTRL	; do it

.Jcp_exit:
	rts			; end of i2s data on/off subroutine

;				*****************************************
;				* Use service mode to debug drive	*
;				*****************************************
Service:			; some code to test service modes
	move.w	#$F001,DS_DATA	; kick into service mode
	bsr	DSA_rx
	move.w	#$F301,DS_DATA	; turn on laser, try to focus it, run motor
	bsr	DSA_rx
;	move.w	#$F000,DS_DATA	; turn off service mode
	rts

;				*****************************************
;				* Command CD to spin-up			*
;				*****************************************
Spin_CD:			; spin-up command
	cmp.l	#0,d1		; check for user-selection of mode
	bne	.sessions	; not 0, jump to multi-session mode

.single:			; start-up normal disc
	move.w	#$1800,DS_DATA	; Spins up on first TOC
	bsr	DSA_rx
	rts			; end of spin-up-normal command

.sessions:			; start-up multi-session on specified session
	or.w	#$1800,d1	; session number must be in d1
	move.w	d1,DS_DATA
	bsr	DSA_rx
	rts			; end of spin-up-multi-session command

;				*****************************************
;				* Command CD to play in 1 of 3 forms	*
;				*****************************************
				; 3-way switch for Play, Play @, & Play A-B
Play_CD:			; call play command with track number in d1
	cmp.l	#0,d1		; see if user has cleared d1
	bne	.playtrak	; not 0, ie. has track number, so go play it

.maybe_AB:
	cmp.l	#0,d3		; if d3 has target value, user wants A-B play
	bne	.AB_play	; branch to A-B play mode

.gotimeCD:			; d3 was 0 so play from user's time-code setup
	move.l	d2,d4		; safe copy of whole start time in d4
	and.l	#$00FF0000,d2	; mask for minutes
	ror.l	#8,d2		; shift the byte over
	ror.l	#8,d2
	or.w	#$1000,d2	; format it for goto
	move.w	d2,DS_DATA	; DSA tx
	bsr	DSA_tx		; wait till sent before sending next 1

	move.l	d4,d2		; bring in copy of original user value of d2
	and.l	#$0000FF00,d2	; mask for seconds
	ror.l	#8,d2
	or.w	#$1100,d2	; format it for goto
	move.w	d2,DS_DATA	; DSA tx
	bsr	DSA_tx		; wait till sent before sending next 1

	move.l	d4,d2		; bring in copy of original user value of d2
	and.l	#$000000FF,d2	; mask for frames
	or.w	#$1200,d2	; format it for goto
	move.w	d2,DS_DATA	; DSA tx
	bsr	DSA_rx		; verify receipt 
	rts			; end of goto time and play command

.playtrak:			; play a track 
	or.w	#$0100,d1	; no pause, format for DSA play-track command
	move.w	d1,DS_DATA	; DSA TX command to play track specified
	bsr	DSA_rx		; get response
	rts			; end of play-track

.AB_play:
	move.l	d2,d4		; safe copy of whole start time in d4
	and.l	#$00FF0000,d2	; mask for minutes
	ror.l	#8,d2		; shift the byte over
	ror.l	#8,d2
	or.w	#$2000,d2	; format it for play A-B
	move.w	d2,DS_DATA	; DSA tx
	bsr	DSA_tx		; wait till sent before sending next 1

	move.l	d4,d2		; bring in copy of original user value of d2
	and.l	#$0000FF00,d2	; mask for seconds
	ror.l	#8,d2
	or.w	#$2100,d2	; format it for play A-B
	move.w	d2,DS_DATA	; DSA tx
	bsr	DSA_tx		; wait till sent before sending next 1

	move.l	d4,d2		; bring in copy of original user value of d2
	and.l	#$000000FF,d2	; mask for frames
	or.l	#$2200,d2	; format it for play A-B
	move.w	d2,DS_DATA	; DSA tx
	bsr	DSA_tx		; wait till sent before sending next 1

	move.l	d3,d4		; safe copy of whole stop time in d4
	and.l	#$00FF0000,d3	; mask for minutes
	ror.l	#8,d3		; shift the byte over
	ror.l	#8,d3
	or.w	#$2300,d3	; format it for play A-B
	move.w	d3,DS_DATA	; DSA tx
	bsr	DSA_tx		; wait till sent before sending next 1

	move.l	d4,d3		; bring in copy of original user value of d3
	and.l	#$0000FF00,d3	; mask for seconds
	ror.l	#8,d3
	or.w	#$2400,d3	; format it for play A-B
	move.w	d3,DS_DATA	; DSA tx
	bsr	DSA_tx		; wait till sent before sending next 1

	move.l	d4,d3		; bring in copy of original user value of d3
	and.l	#$000000FF,d3	; mask for frames
	or.l	#$2500,d3	; format it for play A-B
	move.w	d3,DS_DATA	; DSA tx
	bsr	DSA_rx		; verify receipt 
	rts			; end of Play A_B command

;				*****************************************
;				* Command CD to stop			*
;				*****************************************
Stop_CD:			
	move.w	#$0200,DS_DATA	; DSA TX command to stop
	bsr	DSA_rx
	rts			; end of stop command

;				*****************************************
;				* Command CD to mute the decoder	*
;				*****************************************
Mute_CD:			
	move.w	#$5100,DS_DATA	; DSA TX command to mute
	bsr	DSA_rx
	rts			; end of mute command

;				*****************************************
;				* Command CD to un-mute the decoder	*
;				*****************************************
UnmuteCD:
	move.w	#$51FF,DS_DATA	; DSA TX command to un-mute
	bsr	DSA_rx
	rts			; end of un-mute command

;				*****************************************
;				* Command CD to pause			*
;				*****************************************
Pause_CD:			
	move.w	#$0400,DS_DATA	; DSA TX command to pause
	bsr	DSA_rx
	rts			; end of pause command

;				*****************************************
;				* Command CD to un-pause		*
;				*****************************************
UnpausCD:
	move.w	#$0500,DS_DATA	; DSA TX command to un-pause
	bsr	DSA_rx
	rts			; end of un-pause command

;				*****************************************
;				* Command CD to send ABS timecode	*
;				*****************************************
Get_time:			; send-absolute-timecode command
	move.w	#$0D00,DS_DATA
	bsr	DSA_rx		; receive 3 times to get min:sec:frm
	bsr	DSA_rx
	bsr	DSA_rx
	rts			; end of get-time command

;				*****************************************
;				* Command CD to send no. of sessions	*
;				*****************************************
Seshn_CD:			; send the number of sessions on disc
	move.w	#$5400,DS_DATA
	bsr	DSA_rx
	rts			; end of get-sessions command


;				*****************************************
;				* Command CD to kill its A-B flags	*
;				*****************************************
KillCDAB:
	move.w	#$2600,DS_DATA
	bsr	DSA_rx
	rts			; end of kill A-B flags command

;				*****************************************
;				* Command CD to search forward		*
;				*****************************************
ForwrdCD:			; low byte of d4 sets search mode
	or.l	#$0600,d4	; gets user's search mode byte
	move.w	d4,DS_DATA	; DSA TX command to search
	bsr	DSA_rx
	rts			; end of search command

;				*****************************************
;				* Command CD to search backward		*
;				*****************************************
BckwrdCD:			; low byte of d4 sets search mode
	or.l	#$0700,d4	; gets user's search mode byte
	move.w	d4,DS_DATA	; DSA TX command to search
	bsr	DSA_rx
	rts			; end of search command

;				*****************************************
;				* Command CD to stop searching		*
;				*****************************************
NosrchCD:			
	move.w	#$0800,DS_DATA	; DSA TX command to stop searching
	bsr	DSA_rx
	rts			; end of stop-search command

;				*****************************************
;				* Command CD to send TOC min-max	*
;				*****************************************
CDminmax:			; returns first, last cut nums & endtime
	or.l	#$0300,d4	; get user's session number
	move.w	d4,DS_DATA
	bsr	DSA_rx		; gets min cut number
	bsr	DSA_rx		; gets max cut number
	bsr	DSA_rx		; gets end min
	bsr	DSA_rx		; gets end sec
	bsr	DSA_rx		; gets end frame
	rts			; end of CDminmax command

;				*****************************************
;				* Command CD to send whole TOC		*
;				*****************************************
CDbigTOC:			; gets the entire table of contents
	move.l	#CD_TOC,a0	; setup address of start of TOC storage area
	or.l	#$1400,d4	; get user's session number
	move.w	d4,DS_DATA	; note that CDminmax must be called first

.GetItem:			; receive a TOC entry
	bsr	DSA_rx		; gets cut number
	move.l	CD_wrd0,d1	; the word just fetched from DSA Bus
	and.l	#$FF,d1		; mask for low byte
	move.l	CDTOCend,d2	; get highest cut number known to be on CD
	and.l	#$FF,d2		; mask for low byte
	cmp.l	d1,d2		; compare 'em
	beq	.EnufTOC	; we're done when they are equal
	bsr	DSA_rx		; gets control and address byte
	bsr	DSA_rx		; gets start min of cut
	bsr	DSA_rx		; gets start sec
	bsr	DSA_rx		; gets start frame

.SaveItem:			; Save another valid TOC entry to memory
	move.l	CDcutnum,(a0)+
	move.l	CDcutCAB,(a0)+
	move.l	CDcutstm,(a0)+
	move.l	CDcutsts,(a0)+
	move.l	CDcutstf,(a0)+
	bra	.GetItem	; go get the next TOC entry

.EnufTOC:			; get last item, and exit
	move.l	CDcutnum,(a0)+
	move.l	CDcutCAB,(a0)+
	move.l	CDcutstm,(a0)+
	move.l	CDcutsts,(a0)+
	move.l	CDcutstf,(a0)+
	rts			; end of CDbigTOC command

;				*****************************************
;				* Command CD to send status of disc	*
;				*****************************************
Stat_CD:			
	move.w	#$5000,DS_DATA
	bsr	DSA_rx
	rts			; end of send-disc-status command

;				*****************************************
;				* Command CD to send cut length		*
;				*****************************************
CtlnthCD:			; low byte of d4 must have cut $number
	or.l	#$0900,d4	; gets user's cut number byte
	move.w	d4,DS_DATA	; DSA TX command to search
	bsr	DSA_rx		; do rx twice to get low byte, high byte
	bsr	DSA_rx		; the value of the word is $seconds
	rts			; end of get-cut-length command

;				*****************************************
;				* DSA error handler			*
;				*****************************************
GetCDerr:			; gets & classifies most recent error
	move.w	#$1600,DS_DATA	; request error code
	bsr	DSA_rx		; receive it
	move.l	CD_error,d1	; analyze it
	and.l	#$FF,d1		; low byte only, please
	cmp.l	#0,d1		; if it's 0 there's no error...
	beq	.Clearerr	; so blow out of here, otherwise...
	cmp.l	#$1F,d1		; see if its fatal or not
	bge	.Retry_OK	; not fatal

.Fatals:
	move.l	CDfatals,d0	; get current number of fatal errors
	add.l	#1,d0		; increment
	move.l	d0,CDfatals	; store it
	bra	.Clearerr	; done
	
.Retry_OK:
	move.l	CDmaybes,d0	; get current number of retry-able errors
	add.l	#1,d0		; increment
	move.l	d0,CDmaybes	; store it

.Clearerr:
	move.w	#$1700,DS_DATA	; ask CD to clear error its register
	bsr	DSA_rx
	rts			; end of DSA error handler

;				*****************************************
;				* Transmit a DSA Bus command		*
;				*****************************************
DSA_tx:				; set up as a polling loop
	move.l	BUTCH,d1	; get Butch's ICR into d1
	and.l	#$1000,d1	; mask for d12, DSA TX Intr. pending
	beq	DSA_tx		; nothing here yet, so wait for bit to set 
	move.w	DS_DATA,d1	; flag set, so get the word from Butch
	move.l	d1,CD_wrd0	; store it where Sort can find it
	move.l	DSCNTRL,d5	; read here to clear interrupt flag
	bra	sort_CD		; parse the DSA response

;				*****************************************
;				* Receive DSA Bus response & parse it	*
;				*****************************************
DSA_rx:				; set up as a polling loop
	move.l	BUTCH,d1	; get Butch's ICR into d1
	and.l	#$2000,d1	; mask for d13, DSA RX Intr. pending
	beq	DSA_rx		; nothing here yet, so wait for bit to set 
	move.w	DS_DATA,d1	; flag set, so get the word from Butch
	move.l	d1,CD_wrd0	; store it where we can see it for debugging
	move.l	DSCNTRL,d5	; read here to clear interrupt flag
;	bra	.Endsort	; un-comment to skip the sorting business

sort_CD:			; the commands are decoded per DSA spec.
	move.l	CD_wrd0,d1	; load data word 
	move.l	d1,d2		; copy it for debugging
	andi.l	#$FF00,d1	; mask to see hi byte only

.cmnd_01:			; parse it
	cmp.l	#$0100,d1	; test
	beq	.Found		; found "found" message

.cmnd_03:			; parse it
	cmp.l	#$0300,d1	; test
	beq	.Discstat	; found "disc status" message

.cmnd_04:			; parse it
	cmp.l	#$0400,d1	; test
	beq	.Error		; found "error" message

.cmnd_09:			; parse it
	cmp.l	#$0900,d1	; test
	beq	.CutlenL	; found "cut-length low byte of seconds" 

.cmnd_0A:			; parse it
	cmp.l	#$0A00,d1	; test
	beq	.CutlenH	; found "cut-length high byte of seconds"

.cmnd_10:			; parse it
	cmp.l	#$1000,d1	; test
	beq	.Track		; found "track number"

.cmnd_11:
	cmp.l	#$1100,d1	; test
	beq	.Index		; found "index"

.cmnd_12:
	cmp.l	#$1200,d1	; test
	beq	.Rel_mins	; found "rel. minutes"

.cmnd_13:
	cmp.l	#$1300,d1	; test
	beq	.Rel_secs	; found "rel. seconds"

.cmnd_14:
	cmp.l	#$1400,d1	; test
	beq	.Abs_mins	; found "abs. minutes"

.cmnd_15:
	cmp.l	#$1500,d1	; test
	beq	.Abs_secs	; found "abs. seconds"

.cmnd_16:
	cmp.l	#$1600,d1	; test
	beq	.Frames		; found "frames"

.cmnd_17:
	cmp.l	#$1700,d1	; test
	beq	.Mode		; found "mode"

.cmnd_20:
	cmp.l	#$2000,d1	; test
	beq	.Firs_trk	; found "first track number in TOC"

.cmnd_21:
	cmp.l	#$2100,d1	; test
	beq	.Last_trk	; found "last track number in TOC"

.cmnd_22:
	cmp.l	#$2200,d1	; test
	beq	.End_mins	; found "last minute of last track in this TOC"

.cmnd_23:
	cmp.l	#$2300,d1	; test
	beq	.End_secs	; found "last second of last track in this TOC"

.cmnd_24:
	cmp.l	#$2400,d1	; test
	beq	.Endframe	; found "last frame of last track in this TOC"

.cmnd_54:
	cmp.l	#$5400,d1	; test
	beq	.Sessions	; found "Number of Sessions on disc message"

.cmnd_60:
	cmp.l	#$6000,d1	; test
	beq	.Cutnum		; found "TOC cut number"

.cmnd_61:
	cmp.l	#$6100,d1	; test
	beq	.CutCAB		; found "TOC cut control and address byte"

.cmnd_62:
	cmp.l	#$6200,d1	; test
	beq	.Cutstm		; found "TOC cut start minute"

.cmnd_63:
	cmp.l	#$6300,d1	; test
	beq	.Cutsts		; found "TOC cut start sec"

.cmnd_64:
	cmp.l	#$6400,d1	; test
	beq	.Cutstf		; found "TOC cut start frame"

.cmnd_70:
	cmp.l	#$7000,d1	; test
	beq	.DAC		; found "DAC setting"
	bra	.Endsort	; no command number match - fell thru to here

.Found:
	move.l	d2,CD_found
	bra	.Endsort

.Discstat:
	move.l	d2,CD_stat
	bra	.Endsort

.Error:
	move.l	d2,CD_error
	bra	.Endsort

.CutlenL:
	move.l	d2,CDcutlnL
	bra	.Endsort

.CutlenH:
	move.l	d2,CDcutlnH
	bra	.Endsort

.Track:				
	move.l	d2,CD_track
	bra	.Endsort		

.Index:
	move.l	d2,CD_index
	bra	.Endsort

.Rel_mins:			
	move.l	d2,CD_Rmins
	bra	.Endsort

.Rel_secs:			
	move.l	d2,CD_Rsecs
	bra	.Endsort

.Abs_mins:
	move.l	d2,CD_Amins	
	bra	.Endsort

.Abs_secs:
	move.l	d2,CD_Asecs
	bra	.Endsort

.Frames:
	move.l	d2,CD_Afrms
	bra	.Endsort

.Mode:
	move.l	d2,CD_setup
	bra	.Endsort

.Firs_trk:
	move.l	d2,CDTOCbeg
	bra	.Endsort

.Last_trk:
	move.l	d2,CDTOCend
	bra	.Endsort

.End_mins:
	move.l	d2,CDTOCmin
	bra	.Endsort

.End_secs:
	move.l	d2,CDTOCsec
	bra	.Endsort

.Endframe:
	move.l	d2,CDTOCfrm
	bra	.Endsort

.Sessions:
	move.l	d2,CDseshns
	bra	.Endsort

.Cutnum:
	move.l	d2,CDcutnum
	bra	.Endsort

.CutCAB:
	move.l	d2,CDcutCAB
	bra	.Endsort

.Cutstm:
	move.l	d2,CDcutstm
	bra	.Endsort

.Cutsts:
	move.l	d2,CDcutsts
	bra	.Endsort

.Cutstf:
	move.l	d2,CDcutstf

.DAC:				; now copy the value to named storage 
	move.l	d2,CD_DAC
	bra	.Endsort

.Endsort:
	rts			; end of sorting routine

;**************************************************************************
;**************************************************************************

;myCD_TOC:	dc.l	 0,  1, 65	; Track  1
;		dc.l	 0, 52, 07	; Track  2
;		dc.l	 0, 53, 72	; Track  3
;		dc.l	 1, 54, 21	; Track  4
;		dc.l	 1, 56, 11	; Track  5
;		dc.l	 2, 51, 24	; Track  6
;		dc.l	 2, 53, 14	; Track  7
;		dc.l	 3, 52, 64	; Track  8
;		dc.l	 3, 54, 54	; Track  9
;		dc.l	 5,  1, 67	; Track 10
;		dc.l	 5,  3, 57	; Track 11
;		dc.l	 5, 47, 07	; Track 12
;		dc.l	 5, 48, 72	; Track 13
;		dc.l	 6, 47, 34	; Track 14
;		dc.l	 6, 49, 24	; Track 15
;		dc.l	 7, 32, 72	; Track 16
;		dc.l	 7, 34, 62	; Track 17
;		dc.l	 8, 39, 42	; Track 18
;		dc.l	 8, 41, 32	; Track 19
;		dc.l	 9, 49, 20	; Track 20
;		dc.l	 9, 51, 10	; Track 21
;		dc.l	10, 42, 56	; Track 22
;		dc.l	10, 44, 46	; Track 23
;		dc.l	11, 42, 29	; Track 24
;		dc.l	11, 44, 19	; Track 25
;		dc.l	12, 39, 66	; Track 26
;		dc.l	12, 41, 56	; Track 27
;		dc.l	13, 36, 16	; Track 28
;		dc.l	13, 38,  6	; Track 29
;		dc.l	14, 33,  8	; Track 30
;		dc.l	14, 34, 73	; Track 31
;		dc.l	15, 17, 63	; Track 32

myCD_TOC:	dc.l	 0,  6,  0	; Track  1
		dc.l	 0, 58, 19	; Track  2
		dc.l	 1,  4, 19	; Track  3
		dc.l	 2,  6, 45	; Track  4
		dc.l	 2, 12, 45	; Track  5
		dc.l	 3,  9, 60	; Track  6
		dc.l	 3, 15, 60	; Track  7
		dc.l	 4, 17, 37	; Track  8
		dc.l	 4, 23, 37	; Track  9
		dc.l	 5, 32, 52	; Track 10
		dc.l	 5, 38, 52	; Track 11
		dc.l	 6, 24, 4	; Track 12
		dc.l	 6, 30, 4	; Track 13
		dc.l	 7, 30, 43	; Track 14
		dc.l	 7, 36, 43	; Track 15
		dc.l	 8, 22, 18	; Track 16
		dc.l	 8, 28, 18	; Track 17
		dc.l	 9, 35, 0	; Track 18
		dc.l	 9, 41, 0	; Track 19
		dc.l	10, 50, 65	; Track 20
		dc.l	10, 56, 65	; Track 21
		dc.l	11, 50, 38	; Track 22
		dc.l	11, 56, 38	; Track 23
		dc.l	12, 56, 23	; Track 24
		dc.l	13,  2, 23	; Track 25
		dc.l	13, 59, 72	; Track 26
		dc.l	14,  5, 72	; Track 27
		dc.l	15,  2, 34	; Track 28
		dc.l	15,  8, 34	; Track 29
		dc.l	16,  5, 38	; Track 30
		dc.l	16, 11, 38	; Track 31
		dc.l	16, 56, 30	; Track 32

;**************************************************************************
;**************************************************************************

	.bss

; Butch's registers shadow in DRAM

S_BUTCH :	ds.l	1	; base of Butch=interrupt control register, R/W
SDSCNTRL:	ds.w	3	; DSA control register, R/W
SDS_DATA:	ds.w	3	; DSA TX/RX data, R/W
SI2CNTRL:	ds.l	1	; i2s bus control register, R/W
SSBCNTRL:	ds.l	1	; CD subcode control register, R/W
SSUBDATA:	ds.l	1	; Subcode data register A
SSUBDATB:	ds.l	1	; Subcode data register B
SSB_TIME:	ds.l	1	; Subcode time and compare enable (D24)
SI2SDAT1:	ds.l	1	; i2s FIFO data
SI2SDAT2:	ds.l	1	; i2s FIFO data

; CD TOC area (set to arbitrary big size)

CD_TOC:		ds.l	3000

; Different variable storage area for CD handling routines

CD_base:	ds.l	1

CD_found:	ds.l	1
CD_stat:	ds.l	1
CD_error:	ds.l	1
CDcutlnL:	ds.l	1
CDcutlnH:	ds.l	1
CD_track:	ds.l	1
CD_index:	ds.l	1
CD_Rmins:	ds.l	1
CD_Rsecs:	ds.l	1
CD_Amins:	ds.l	1
CD_Asecs:	ds.l	1
CD_Afrms:	ds.l	1
CD_setup:	ds.l	1
CDTOCbeg:	ds.l	1
CDTOCend:	ds.l	1
CDTOCmin:	ds.l	1
CDTOCsec:	ds.l	1
CDTOCfrm:	ds.l	1
CDseshns:	ds.l	1
CDcutnum:	ds.l	1
CDcutCAB:	ds.l	1
CDcutstm:	ds.l	1
CDcutsts:	ds.l	1
CDcutstf:	ds.l	1
CD_DAC:		ds.l	1

CD_wrd0:	ds.l	1

CDfatals:	ds.l	1
CDmaybes:	ds.l	1

; These variables are written to in normal

CD_retry:	ds.l	1
PS_flag	:	ds.l	1

; This group of variables must be initialized.

Init_cut:	ds.l	1
CD_mode	:	ds.l	1
DAC_mode:	ds.l	1

Strt_min:	ds.w	1	; format is $20nn, where nn is $minute value
Strt_sec:	ds.w	1	; format is $21nn, where nn is $second value
strt_frm:	ds.w	1	; format is $22nn, where nn is $frame value
Stop_min:	ds.w	1	; format is $23nn, where nn is $minute value
Stop_sec:	ds.w	1	; format is $24nn, where nn is $second value
Stop_frm:	ds.w	1	; format is $25nn, where nn is $frame value

; More CD inits that are required for DSA

Waittime:	ds.w	1	; hex value to use in wait loop
Session :	ds.w	1	; Session number to spin-up on

; DRAM memory area where the incoming CD data is kept
; These define the main DRAM CD data buf area
; edit the values both here & in the .GAS files

Blok_ptr:	ds.l	1	; pointer into CD data block stored here
Blok_beg:	ds.l	1	; start of CD raw data buf
Blok_end:	ds.l	1	; end of CD test buf

;**************************************************************************
;**************************************************************************

	.end
